# Copyright 2021 Troila
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from neutron_lib import constants as lib_const
from neutron_lib import context
from oslo_utils import uuidutils

from neutron.agent.l3 import agent as l3_agent
from neutron.agent.l3 import dvr_edge_router
from neutron.agent.l3 import dvr_local_router as dvr_router
from neutron.agent.l3.extensions import ndp_proxy as np
from neutron.agent.l3 import l3_agent_extension_api as l3_ext_api
from neutron.agent.l3 import router_info
from neutron.agent.linux import iptables_manager
from neutron.api.rpc.callbacks.consumer import registry
from neutron.api.rpc.callbacks import events
from neutron.api.rpc.callbacks import resources
from neutron.api.rpc.handlers import resources_rpc
from neutron.objects import agent as agent_obj
from neutron.objects import ndp_proxy as np_obj
from neutron.objects import ports as ports_obj
from neutron.tests import base
from neutron.tests.unit.agent.l3 import test_agent
from neutron.tests.unit.agent.l3 import test_dvr_local_router

_uuid = uuidutils.generate_uuid

HOSTNAME = 'testhost'


class NDPProxyExtensionTestCaseBase(base.BaseTestCase):

    def setUp(self):
        super(NDPProxyExtensionTestCaseBase, self).setUp()
        self.context = context.get_admin_context()
        self.connection = mock.Mock()
        self.ext_port_id = _uuid()
        self.ex_net_id = _uuid()
        self.ex_gw_port = {'id': self.ext_port_id,
                           'network_id': self.ex_net_id,
                           'gw_port_host': HOSTNAME}
        self.fake_router_id = _uuid()
        self.port_id = _uuid()
        self.agent_api = l3_ext_api.L3AgentExtensionAPI(None, None)
        self.np_ext = np.NDPProxyAgentExtension()
        self.np_ext.consume_api(self.agent_api)
        self.np_ext.initialize(
            self.connection, lib_const.L3_AGENT_MODE)
        self.ndpproxy = np_obj.NDPProxy(
            context=None, id=_uuid(),
            router_id=self.fake_router_id,
            port_id=self.port_id, ip_address='2002::1:3')
        port_binding = ports_obj.PortBinding(port_id=self.port_id,
                                             host=HOSTNAME)
        port_obj = ports_obj.Port(id=self.port_id, bindings=[port_binding])
        self.ndp_proxies = [self.ndpproxy]
        self.ports = [port_obj]
        agent_configurations = {
            'agent_mode': lib_const.L3_AGENT_MODE_DVR_NO_EXTERNAL}
        self.agent_obj = agent_obj.Agent(
            id=_uuid(), host=HOSTNAME,
            agent_type=lib_const.AGENT_TYPE_L3,
            configurations=agent_configurations)
        self.ip_wrapper = mock.patch('neutron.agent.linux.'
                                     'ip_lib.IPWrapper').start()
        self._set_pull_mock()

    def _set_pull_mock(self):

        def _bulk_pull_mock(context, resource_type, filter_kwargs=None):
            if resource_type == resources.PORT:
                return [port for port in self.ports if
                        port.id == filter_kwargs['id']]
            if resource_type == resources.AGENT:
                return [self.agent_obj]
            if resource_type == resources.NDPPROXY:
                result = []
                if 'router_id' in filter_kwargs:
                    for ndp_proxy in self.ndp_proxies:
                        if ndp_proxy.router_id in filter_kwargs['router_id']:
                            result.append(ndp_proxy)
                    return result
                return self.ndp_proxie

        self.pull = mock.patch('neutron.api.rpc.handlers.resources_rpc.'
                               'ResourcesPullRpcApi.pull').start()
        self.bulk_pull = mock.patch('neutron.api.rpc.handlers.resources_rpc.'
                                    'ResourcesPullRpcApi.bulk_pull').start()
        self.bulk_pull.side_effect = _bulk_pull_mock


class NDPProxyExtensionDVRTestCase(
        NDPProxyExtensionTestCaseBase,
        test_dvr_local_router.TestDvrRouterOperations):

    def setUp(self):
        super(NDPProxyExtensionDVRTestCase, self).setUp()
        self.conf.host = HOSTNAME
        self.conf.agent_mode = lib_const.L3_AGENT_MODE_DVR
        self.agent = l3_agent.L3NATAgent(HOSTNAME, self.conf)
        self.add_route = mock.MagicMock()
        self.delete_route = mock.MagicMock()
        mock_route_cmd = mock.MagicMock()
        mock_route_cmd.add_route = self.add_route
        mock_route_cmd.delete_route = self.delete_route
        self.mock_ip_dev.route = mock_route_cmd
        self.lladdr = "fe80::f816:3eff:fe5f:9d67"
        get_ipv6_lladdr = mock.patch("neutron.agent.linux.ip_lib."
                                     "get_ipv6_lladdr").start()
        get_ipv6_lladdr.return_value = "%s/64" % self.lladdr
        self.router = {'id': self.fake_router_id,
                       'gw_port': self.ex_gw_port,
                       'ha': False,
                       'distributed': True,
                       'enable_ndp_proxy': True}
        kwargs = {
            'agent': self.agent,
            'router_id': self.fake_router_id,
            'router': self.router,
            'agent_conf': self.conf,
            'interface_driver': mock.Mock()}
        self.router_info = dvr_router.DvrLocalRouter(HOSTNAME, **kwargs)
        self.get_router_info = mock.patch(
            'neutron.agent.l3.l3_agent_extension_api.'
            'L3AgentExtensionAPI.get_router_info').start()
        self.get_router_info.return_value = self.router_info
        self.router_info.fip_ns = self.agent.get_fip_ns(self.ex_net_id)
        agent_ext_port_id = _uuid()
        self.router_info.fip_ns.agent_gateway_port = {'id': agent_ext_port_id}
        self.namespace = "fip-%s" % self.ex_net_id
        self.agent_ext_dvice = "fg-%s" % agent_ext_port_id[:11]
        self.ip_wrapper.reset_mock()

    def test_create_router(self):
        self.np_ext.add_router(self.context, self.router)
        expected_calls = [
            mock.call('2002::1:3', via=self.lladdr)]
        self.assertEqual(expected_calls, self.add_route.mock_calls)
        sysctl_cmd = ['sysctl', '-w',
                      'net.ipv6.conf.%s.proxy_ndp=1' % self.agent_ext_dvice]
        proxy_cmd = ['ip', '-6', 'neigh', 'add',
                     'proxy', '2002::1:3', 'dev', self.agent_ext_dvice]
        ndsend_cmd = ['ndsend', '2002::1:3', self.agent_ext_dvice]
        expected_calls = [
            mock.call(namespace=self.namespace),
            mock.call().netns.execute(sysctl_cmd, privsep_exec=True),
            mock.call(namespace=self.namespace),
            mock.call().netns.execute(proxy_cmd, privsep_exec=True),
            mock.call().netns.execute(
                ndsend_cmd, check_exit_code=False,
                log_fail_as_error=True, privsep_exec=True),
            mock.call(namespace=self.namespace)]
        self.assertEqual(expected_calls, self.ip_wrapper.mock_calls)

    def test_update_router(self):
        self.np_ext.add_router(self.context, self.router)
        self.add_route.reset_mock()
        self.ip_wrapper.reset_mock()
        self.np_ext.update_router(self.context, self.router)
        self.add_route.assert_not_called()
        sysctl_cmd = ['sysctl', '-w',
                      'net.ipv6.conf.%s.proxy_ndp=1' % self.agent_ext_dvice]
        expected_calls = [
            mock.call(namespace=self.namespace),
            mock.call().netns.execute(sysctl_cmd, privsep_exec=True),
            mock.call(namespace=self.namespace)]
        self.assertEqual(expected_calls, self.ip_wrapper.mock_calls)

    def test_add_ndp_proxy_update_router(self):
        self.np_ext.add_router(self.context, self.router)
        self.add_route.reset_mock()
        self.ip_wrapper.reset_mock()
        ndpproxy = np_obj.NDPProxy(
            context=None, id=_uuid(),
            router_id=self.fake_router_id,
            port_id=self.port_id, ip_address='2002::1:6')
        self.ndp_proxies.append(ndpproxy)
        self.np_ext.update_router(self.context, self.router)
        expected_calls = [
            mock.call('2002::1:6', via=self.lladdr)]
        self.assertEqual(expected_calls, self.add_route.mock_calls)
        sysctl_cmd = ['sysctl', '-w',
                      'net.ipv6.conf.%s.proxy_ndp=1' % self.agent_ext_dvice]
        proxy_cmd = ['ip', '-6', 'neigh', 'add',
                     'proxy', '2002::1:6', 'dev', self.agent_ext_dvice]
        ndsend_cmd = ['ndsend', '2002::1:6', self.agent_ext_dvice]
        expected_calls = [
            mock.call(namespace=self.namespace),
            mock.call().netns.execute(sysctl_cmd, privsep_exec=True),
            mock.call(namespace=self.namespace),
            mock.call().netns.execute(proxy_cmd, privsep_exec=True),
            mock.call().netns.execute(
                ndsend_cmd, check_exit_code=False,
                log_fail_as_error=True, privsep_exec=True),
            mock.call(namespace=self.namespace)]
        self.assertEqual(expected_calls, self.ip_wrapper.mock_calls)

    def test_del_ndp_proxy_update_router(self):
        self.np_ext.add_router(self.context, self.router)
        self.add_route.reset_mock()
        self.ip_wrapper.reset_mock()
        self.ndp_proxies = []
        self.np_ext.update_router(self.context, self.router)
        self.add_route.assert_not_called()
        expected_calls = [
            mock.call('2002::1:3', via=self.lladdr)]
        self.assertEqual(expected_calls, self.delete_route.mock_calls)
        sysctl_cmd = ['sysctl', '-w',
                      'net.ipv6.conf.%s.proxy_ndp=1' % self.agent_ext_dvice]
        proxy_cmd = ['ip', '-6', 'neigh', 'del',
                     'proxy', '2002::1:3', 'dev', self.agent_ext_dvice]
        expected_calls = [
            mock.call(namespace=self.namespace),
            mock.call().netns.execute(sysctl_cmd, privsep_exec=True),
            mock.call(namespace=self.namespace),
            mock.call().netns.execute(proxy_cmd, privsep_exec=True)]
        self.assertEqual(expected_calls, self.ip_wrapper.mock_calls)

    def test__handle_notification(self):
        self.np_ext.add_router(self.context, self.router)
        self.add_route.reset_mock()
        self.ip_wrapper.reset_mock()
        ndpproxy = np_obj.NDPProxy(
            context=None, id=_uuid(),
            router_id=self.fake_router_id,
            port_id=self.port_id, ip_address='2002::1:5')
        self.np_ext._handle_notification(mock.MagicMock(), mock.MagicMock(),
                                         [ndpproxy], events.CREATED)
        expected_calls = [
            mock.call('2002::1:5', via=self.lladdr)]
        self.assertEqual(expected_calls, self.add_route.mock_calls)
        proxy_cmd = ['ip', '-6', 'neigh', 'add',
                     'proxy', '2002::1:5', 'dev', self.agent_ext_dvice]
        ndsend_cmd = ['ndsend', '2002::1:5', self.agent_ext_dvice]
        expected_calls = [
            mock.call(namespace=self.namespace),
            mock.call().netns.execute(proxy_cmd, privsep_exec=True),
            mock.call().netns.execute(
                ndsend_cmd, check_exit_code=False,
                log_fail_as_error=True, privsep_exec=True)]
        self.assertEqual(expected_calls, self.ip_wrapper.mock_calls)
        self.add_route.reset_mock()
        self.ip_wrapper.reset_mock()
        self.np_ext._handle_notification(mock.MagicMock(), mock.MagicMock(),
                                         [ndpproxy], events.DELETED)
        self.add_route.assert_not_called()
        expected_calls = [
            mock.call('2002::1:5', via=self.lladdr)]
        self.assertEqual(expected_calls, self.delete_route.mock_calls)
        proxy_cmd = ['ip', '-6', 'neigh', 'del',
                     'proxy', '2002::1:5', 'dev', self.agent_ext_dvice]
        expected_calls = [
            mock.call(namespace=self.namespace),
            mock.call().netns.execute(proxy_cmd, privsep_exec=True)]
        self.assertEqual(expected_calls, self.ip_wrapper.mock_calls)


class NDPProxyExtensionLegacyDVRNoExternalTestCaseBase(
        NDPProxyExtensionTestCaseBase,
        test_agent.BasicRouterOperationsFramework):

    def _mock_iptables_actions(self):
        self.get_router_info = mock.patch(
            'neutron.agent.l3.l3_agent_extension_api.'
            'L3AgentExtensionAPI.get_router_info').start()
        self.add_chain = mock.patch('neutron.agent.linux.iptables_manager.'
                                    'IptablesTable.add_chain').start()
        self.remove_chain = mock.patch('neutron.agent.linux.iptables_manager.'
                                       'IptablesTable.remove_chain').start()
        self.add_rule = mock.patch('neutron.agent.linux.iptables_manager.'
                                   'IptablesTable.add_rule').start()
        self.remove_rule = mock.patch('neutron.agent.linux.iptables_manager.'
                                      'IptablesTable.remove_rule').start()

        def _add_chain_mock(chain):
            self.iptables_manager.ipv6[
                'filter'].chains.append(chain)

        def _remove_chain_mock(chain):
            self.iptables_manager.ipv6[
                'filter'].chains.remove(chain)

        def _add_rule_mock(chain, rule, top=False):
            rule_obj = mock.MagicMock()
            rule_obj.rule = rule
            self.iptables_manager.ipv6[
                'filter'].rules.append(rule_obj)

        def _remove_rule_mock(chain, rule, top=False):
            for rule_obj in self.iptables_manager.ipv6[
                    'filter'].rules:
                if rule == rule_obj.rule:
                    self.iptables_manager.ipv6[
                        'filter'].rules.remove(rule_obj)
                    break

        self.get_router_info.return_value = self.router_info
        self.add_chain.side_effect = _add_chain_mock
        self.remove_chain.side_effect = _remove_chain_mock
        self.add_rule.side_effect = _add_rule_mock
        self.remove_rule.side_effect = _remove_rule_mock

    def _test_create_router(self):
        self.np_ext.add_router(self.context, self.router)
        expected_calls = [mock.call(np.DEFAULT_NDP_PROXY_CHAIN)]
        self.assertEqual(expected_calls, self.add_chain.mock_calls)
        default_rule = '-i %s -j DROP' % self.ext_device_name
        subnet_rule = ('-i %s --destination %s -j %s-%s') % (
            self.ext_device_name, '2001::1:0/112', self.wrap_name,
            np.DEFAULT_NDP_PROXY_CHAIN)
        accept_rule = '-i %s --destination %s -j ACCEPT' % (
            self.ext_device_name, '2002::1:3')
        expected_calls = [
            mock.call(np.DEFAULT_NDP_PROXY_CHAIN, default_rule),
            mock.call('FORWARD', subnet_rule),
            mock.call(np.DEFAULT_NDP_PROXY_CHAIN, accept_rule, top=True)]
        self.assertEqual(expected_calls, self.add_rule.mock_calls)
        sysctl_cmd = ['sysctl', '-w',
                      'net.ipv6.conf.%s.proxy_ndp=1' % self.ext_device_name]
        proxy_cmd = ['ip', '-6', 'neigh', 'add', 'proxy',
                     '2002::1:3', 'dev', self.ext_device_name]
        ndsend_cmd = ['ndsend', '2002::1:3', self.ext_device_name]
        expected_calls = [
            mock.call(namespace=self.namespace),
            mock.call().netns.execute(sysctl_cmd, privsep_exec=True),
            mock.call(namespace=self.namespace),
            mock.call().netns.execute(proxy_cmd, privsep_exec=True),
            mock.call().netns.execute(ndsend_cmd, check_exit_code=False,
                                      log_fail_as_error=True,
                                      privsep_exec=True),
            mock.call(namespace=self.namespace)]
        self.assertEqual(expected_calls, self.ip_wrapper.mock_calls)

    def _test_update_router(self):
        self.np_ext.add_router(self.context, self.router)
        self.add_chain.reset_mock()
        self.add_rule.reset_mock()
        self.ip_wrapper.reset_mock()
        self.np_ext.update_router(self.context, self.router)
        self.add_chain.assert_not_called()
        self.add_rule.assert_not_called()
        sysctl_cmd = ['sysctl', '-w',
                      'net.ipv6.conf.%s.proxy_ndp=1' % self.ext_device_name]
        expected_calls = [
            mock.call(namespace=self.namespace),
            mock.call().netns.execute(sysctl_cmd, privsep_exec=True),
            mock.call(namespace=self.namespace),
            mock.call(namespace=self.namespace)]
        self.assertEqual(expected_calls, self.ip_wrapper.mock_calls)

    def _test_add_ndp_proxy_update_router(self):
        self.np_ext.add_router(self.context, self.router)
        self.add_chain.reset_mock()
        self.add_rule.reset_mock()
        self.ip_wrapper.reset_mock()
        self.ndp_proxies.append(
            np_obj.NDPProxy(context=None, id=_uuid(),
                            router_id=self.fake_router_id,
                            port_id=self.port_id,
                            ip_address='2002::1:4'))
        self.np_ext.update_router(self.context, self.router)
        self.add_chain.assert_not_called()
        accept_rule = '-i %s --destination %s -j ACCEPT' % (
            self.ext_device_name, '2002::1:4')
        expected_calls = [
            mock.call(np.DEFAULT_NDP_PROXY_CHAIN, accept_rule, top=True)]
        self.assertEqual(expected_calls, self.add_rule.mock_calls)
        sysctl_cmd = ['sysctl', '-w',
                      'net.ipv6.conf.%s.proxy_ndp=1' % self.ext_device_name]
        proxy_cmd = ['ip', '-6', 'neigh', 'add', 'proxy',
                     '2002::1:4', 'dev', self.ext_device_name]
        ndsend_cmd = ['ndsend', '2002::1:4', self.ext_device_name]
        expected_calls = [
            mock.call(namespace=self.namespace),
            mock.call().netns.execute(sysctl_cmd, privsep_exec=True),
            mock.call(namespace=self.namespace),
            mock.call().netns.execute(proxy_cmd, privsep_exec=True),
            mock.call().netns.execute(ndsend_cmd, check_exit_code=False,
                                      log_fail_as_error=True,
                                      privsep_exec=True),
            mock.call(namespace=self.namespace)]
        self.assertEqual(expected_calls, self.ip_wrapper.mock_calls)

    def _test_del_ndp_proxy_update_router(self):
        self.np_ext.add_router(self.context, self.router)
        self.add_chain.reset_mock()
        self.add_rule.reset_mock()
        self.ip_wrapper.reset_mock()
        self.ndp_proxies = []
        self.np_ext.update_router(self.context, self.router)
        self.add_chain.assert_not_called()
        self.remove_chain.assert_not_called()
        self.add_rule.assert_not_called()
        accept_rule = '-i %s --destination %s -j ACCEPT' % (
            self.ext_device_name, '2002::1:3')
        expected_calls = [mock.call(np.DEFAULT_NDP_PROXY_CHAIN,
                                    accept_rule, top=True)]
        self.assertEqual(expected_calls, self.remove_rule.mock_calls)
        sysctl_cmd = ['sysctl', '-w',
                      'net.ipv6.conf.%s.proxy_ndp=1' % self.ext_device_name]
        proxy_cmd = ['ip', '-6', 'neigh', 'del',
                     'proxy', '2002::1:3', 'dev', self.ext_device_name]
        expected_calls = [
            mock.call(namespace=self.namespace),
            mock.call().netns.execute(sysctl_cmd, privsep_exec=True),
            mock.call(namespace=self.namespace),
            mock.call(namespace=self.namespace),
            mock.call().netns.execute(proxy_cmd, privsep_exec=True)]
        self.assertEqual(expected_calls, self.ip_wrapper.mock_calls)

    def _test_add_subnet_update_router(self):
        self.np_ext.add_router(self.context, self.router)
        self.add_chain.reset_mock()
        self.add_rule.reset_mock()
        self.ip_wrapper.reset_mock()
        self.internal_ports.append(
            {'subnets': [{'cidr': '2001::5:0/112'}]})
        self.np_ext.update_router(self.context, self.router)
        self.add_chain.assert_not_called()
        self.remove_chain.assert_not_called()
        subnet_rule = ('-i %s --destination 2001::5:0/112 -j %s-%s') % (
            self.ext_device_name, self.wrap_name,
            np.DEFAULT_NDP_PROXY_CHAIN)
        expected_calls = [mock.call('FORWARD', subnet_rule)]
        self.assertEqual(expected_calls, self.add_rule.mock_calls)
        self.remove_rule.assert_not_called()
        sysctl_cmd = ['sysctl', '-w',
                      'net.ipv6.conf.%s.proxy_ndp=1' % self.ext_device_name]
        expected_calls = [
            mock.call(namespace=self.namespace),
            mock.call().netns.execute(sysctl_cmd, privsep_exec=True),
            mock.call(namespace=self.namespace),
            mock.call(namespace=self.namespace)]
        self.assertEqual(expected_calls, self.ip_wrapper.mock_calls)

    def _test_remove_subnet_update_router(self):
        self.np_ext.add_router(self.context, self.router)
        self.add_chain.reset_mock()
        self.add_rule.reset_mock()
        self.ip_wrapper.reset_mock()
        self.router_info.internal_ports = []
        self.np_ext.update_router(self.context, self.router)
        self.add_chain.assert_not_called()
        self.remove_chain.assert_not_called()
        self.add_rule.assert_not_called()
        subnet_rule = ('-i %s --destination %s -j %s-%s') % (
            self.ext_device_name, '2001::1:0/112', self.wrap_name,
            np.DEFAULT_NDP_PROXY_CHAIN)
        expected_calls = [mock.call('FORWARD', subnet_rule)]
        self.assertEqual(expected_calls, self.remove_rule.mock_calls)
        sysctl_cmd = ['sysctl', '-w',
                      'net.ipv6.conf.%s.proxy_ndp=1' % self.ext_device_name]
        expected_calls = [
            mock.call(namespace=self.namespace),
            mock.call().netns.execute(sysctl_cmd, privsep_exec=True),
            mock.call(namespace=self.namespace),
            mock.call(namespace=self.namespace)]
        self.assertEqual(expected_calls, self.ip_wrapper.mock_calls)

    def _test_disable_ndp_proxy_update_router(self):
        self.np_ext.add_router(self.context, self.router)
        self.add_chain.reset_mock()
        self.add_rule.reset_mock()
        self.ip_wrapper.reset_mock()
        self.router['enable_ndp_proxy'] = False
        self.np_ext.update_router(self.context, self.router)
        self.add_chain.assert_not_called()
        expected_calls = [mock.call(np.DEFAULT_NDP_PROXY_CHAIN)]
        self.assertEqual(expected_calls, self.remove_chain.mock_calls)
        self.add_rule.assert_not_called()
        self.remove_rule.assert_not_called()
        flush_cmd = ['ip', '-6', 'neigh', 'flush', 'proxy']
        sysctl_cmd = ['sysctl', '-w',
                      'net.ipv6.conf.%s.proxy_ndp=0' % self.ext_device_name]
        expected_calls = [
            mock.call(namespace=self.namespace),
            mock.call().netns.execute(flush_cmd, check_exit_code=False,
                                      privsep_exec=True),
            mock.call().netns.execute(sysctl_cmd, privsep_exec=True)]
        self.assertEqual(expected_calls, self.ip_wrapper.mock_calls)

    def _test__handle_notification(self):
        self.np_ext.add_router(self.context, self.router)
        self.add_chain.reset_mock()
        self.add_rule.reset_mock()
        self.ip_wrapper.reset_mock()
        ndpproxy = np_obj.NDPProxy(
            context=None, id=_uuid(),
            router_id=self.fake_router_id,
            port_id=self.port_id, ip_address='2002::1:5')
        self.np_ext._handle_notification(mock.MagicMock(), mock.MagicMock(),
                                         [ndpproxy], events.CREATED)
        self.add_chain.assert_not_called()
        self.remove_chain.assert_not_called()
        accept_rule = '-i %s --destination %s -j ACCEPT' % (
            self.ext_device_name, '2002::1:5')
        expected_calls = [
            mock.call(np.DEFAULT_NDP_PROXY_CHAIN, accept_rule, top=True)]
        self.assertEqual(expected_calls, self.add_rule.mock_calls)
        self.remove_rule.assert_not_called()
        proxy_cmd = ['ip', '-6', 'neigh', 'add', 'proxy',
                     '2002::1:5', 'dev', self.ext_device_name]
        ndsend_cmd = ['ndsend', '2002::1:5', self.ext_device_name]
        expected_calls = [
            mock.call(namespace=self.namespace),
            mock.call().netns.execute(proxy_cmd, privsep_exec=True),
            mock.call().netns.execute(ndsend_cmd, check_exit_code=False,
                                      log_fail_as_error=True,
                                      privsep_exec=True)]
        self.assertEqual(expected_calls, self.ip_wrapper.mock_calls)
        self.add_chain.reset_mock()
        self.remove_chain.reset_mock()
        self.add_rule.reset_mock()
        self.remove_rule.reset_mock()
        self.ip_wrapper.reset_mock()
        self.np_ext._handle_notification(mock.MagicMock(), mock.MagicMock(),
                                         [ndpproxy], events.DELETED)
        self.add_chain.assert_not_called()
        self.remove_chain.assert_not_called()
        self.add_rule.assert_not_called()
        accept_rule = ('-i %s --destination %s -j ACCEPT') % (
            self.ext_device_name, '2002::1:5')
        expected_calls = [
            mock.call(np.DEFAULT_NDP_PROXY_CHAIN, accept_rule, top=True)]
        self.assertEqual(expected_calls, self.remove_rule.mock_calls)
        proxy_cmd = ['ip', '-6', 'neigh', 'del', 'proxy', '2002::1:5',
                     'dev', self.ext_device_name]
        expected_calls = [
            mock.call(namespace=self.namespace),
            mock.call().netns.execute(proxy_cmd, privsep_exec=True)]
        self.assertEqual(expected_calls, self.ip_wrapper.mock_calls)


class NDPProxyExtensionLegacyTestCase(
        NDPProxyExtensionLegacyDVRNoExternalTestCaseBase):
    def setUp(self):
        super(NDPProxyExtensionLegacyTestCase, self).setUp()
        self.conf.host = HOSTNAME
        self.conf.agent_mode = lib_const.L3_AGENT_MODE_LEGACY
        self.agent = l3_agent.L3NATAgent(HOSTNAME, self.conf)
        self.ext_device_name = 'qg-%s' % self.ext_port_id[0:11]
        self.internal_ports = [{'subnets': [{'cidr': '2001::1:0/112'}]}]
        self.router = {'id': self.fake_router_id,
                       'gw_port': self.ex_gw_port,
                       'ha': False,
                       'distributed': False,
                       'enable_ndp_proxy': True}
        self.router_info = router_info.RouterInfo(
            self.agent, self.fake_router_id, self.router, **self.ri_kwargs)
        self.iptables_manager = self.router_info.iptables_manager
        self.router_info.internal_ports = self.internal_ports
        self.router_info.ex_gw_port = self.ex_gw_port
        self.iptables_manager.ipv6['filter'].chains = []
        self.iptables_manager.ipv6['filter'].rules = []
        self.agent.router_info[self.router['id']] = self.router_info
        self.wrap_name = self.iptables_manager.wrap_name
        self.namespace = "qrouter-" + self.fake_router_id
        self._mock_iptables_actions()
        self.ip_wrapper.reset_mock()

    def test_create_router(self):
        self._test_create_router()

    def test_update_router(self):
        self._test_update_router()

    def test_add_ndp_proxy_update_router(self):
        self._test_add_ndp_proxy_update_router()

    def test_del_ndp_proxy_update_router(self):
        self._test_del_ndp_proxy_update_router()

    def test_add_subnet_update_router(self):
        self._test_add_subnet_update_router()

    def test_remove_subnet_update_router(self):
        self._test_remove_subnet_update_router()

    def test_disable_ndp_proxy_update_router(self):
        self._test_disable_ndp_proxy_update_router()

    def test__handle_notification(self):
        self._test__handle_notification()


class NDPProxyExtensionDVRNoExternalTestCase(
        NDPProxyExtensionLegacyDVRNoExternalTestCaseBase):
    def setUp(self):
        super(NDPProxyExtensionLegacyDVRNoExternalTestCaseBase, self).setUp()
        self.conf.host = HOSTNAME
        self.conf.agent_mode = lib_const.L3_AGENT_MODE_DVR_SNAT
        self.agent = l3_agent.L3NATAgent(HOSTNAME, self.conf)
        self.ext_device_name = 'qg-%s' % self.ext_port_id[0:11]
        self.internal_ports = [{'subnets': [{'cidr': '2001::1:0/112'}]}]
        self.router = {'id': self.fake_router_id,
                       'gw_port': self.ex_gw_port,
                       'gw_port_host': HOSTNAME,
                       'ha': False,
                       'distributed': True,
                       'enable_ndp_proxy': True}
        interface_driver = mock.Mock()
        interface_driver.DEV_NAME_LEN = 14
        kwargs = {
            'agent': self.agent,
            'router_id': self.fake_router_id,
            'router': self.router,
            'agent_conf': self.conf,
            'interface_driver': interface_driver}
        self._mock_load_fip = mock.patch.object(
            dvr_edge_router.DvrEdgeRouter,
            '_load_used_fip_information').start()
        self.router_info = dvr_edge_router.DvrEdgeRouter(HOSTNAME, **kwargs)
        self.iptables_manager = iptables_manager.IptablesManager(
            namespace=self.router_info.snat_namespace.name,
            use_ipv6=self.router_info.use_ipv6)
        self.router_info.snat_iptables_manager = self.iptables_manager
        self.router_info.internal_ports = self.internal_ports
        self.router_info.ex_gw_port = self.ex_gw_port
        self.iptables_manager.ipv6['filter'].chains = []
        self.iptables_manager.ipv6['filter'].rules = []
        self.agent.router_info[self.router['id']] = self.router_info
        self.wrap_name = self.iptables_manager.wrap_name
        self.namespace = "snat-" + self.fake_router_id
        self._mock_iptables_actions()
        self.ip_wrapper.reset_mock()

    def test_create_router(self):
        self._test_create_router()

    def test_update_router(self):
        self._test_update_router()

    def test_add_ndp_proxy_update_router(self):
        self._test_add_ndp_proxy_update_router()

    def test_del_ndp_proxy_update_router(self):
        self._test_del_ndp_proxy_update_router()

    def test_add_subnet_update_router(self):
        self._test_add_subnet_update_router()

    def test_remove_subnet_update_router(self):
        self._test_remove_subnet_update_router()

    def test_disable_ndp_proxy_update_router(self):
        self._test_disable_ndp_proxy_update_router()

    def test__handle_notification(self):
        self._test__handle_notification()


class NDPProxyExtensionInitializeTestCase(NDPProxyExtensionTestCaseBase):

    @mock.patch.object(registry, 'register')
    @mock.patch.object(resources_rpc, 'ResourcesPushRpcCallback')
    def test_initialize_subscribed_to_rpc(self, rpc_mock, subscribe_mock):
        call_to_patch = 'neutron_lib.rpc.Connection'
        with mock.patch(call_to_patch,
                        return_value=self.connection) as create_connection:
            self.np_ext.initialize(
                self.connection, lib_const.L3_AGENT_MODE)
            create_connection.assert_has_calls([mock.call()])
            self.connection.create_consumer.assert_has_calls(
                [mock.call(
                     resources_rpc.resource_type_versioned_topic(
                         resources.NDPPROXY),
                     [rpc_mock()],
                     fanout=True)]
            )
            subscribe_mock.assert_called_with(
                mock.ANY, resources.NDPPROXY)


class RouterNDPProxyMappingTestCase(base.BaseTestCase):
    def setUp(self):
        super(RouterNDPProxyMappingTestCase, self).setUp()
        self.mapping = np.RouterNDPProxyMapping()
        self.router1 = _uuid()
        self.router2 = _uuid()
        self.ndpproxy1 = np_obj.NDPProxy(
            context=None, id=_uuid(),
            router_id=self.router1,
            port_id=_uuid(), ip_address='2002::1:3')
        self.ndpproxy2 = np_obj.NDPProxy(
            context=None, id=_uuid(),
            router_id=self.router2,
            port_id=_uuid(), ip_address='2002::1:4')
        self.ndpproxies = [self.ndpproxy1, self.ndpproxy2]

    def test_set_ndp_proxies(self):
        self.mapping.set_ndp_proxies(self.ndpproxies)
        for ndp_proxy in self.ndpproxies:
            res = self.mapping.get_ndp_proxy(ndp_proxy.id)
            self.assertEqual(ndp_proxy, res)
        router1_ndp_proxies = self.mapping.get_ndp_proxies_by_router_id(
            self.router1)
        self.assertEqual([self.ndpproxy1], router1_ndp_proxies)
        router2_ndp_proxies = self.mapping.get_ndp_proxies_by_router_id(
            self.router2)
        self.assertEqual([self.ndpproxy2], router2_ndp_proxies)

    def test_del_ndp_proxies(self):
        self.mapping.set_ndp_proxies(self.ndpproxies)
        self.mapping.del_ndp_proxies([self.ndpproxy2])
        res = self.mapping.get_ndp_proxy(self.ndpproxy2.id)
        self.assertIsNone(res)
        router1_ndp_proxies = self.mapping.get_ndp_proxies_by_router_id(
            self.router1)
        self.assertEqual([self.ndpproxy1], router1_ndp_proxies)
        router2_ndp_proxies = self.mapping.get_ndp_proxies_by_router_id(
            self.router2)
        self.assertEqual([], router2_ndp_proxies)

    def test_clear_by_router_id(self):
        self.mapping.set_ndp_proxies(self.ndpproxies)
        self.mapping.clear_by_router_id(self.router1)
        np1 = self.mapping.get_ndp_proxy(self.ndpproxy1.id)
        self.assertIsNone(np1)
        np2 = self.mapping.get_ndp_proxy(self.ndpproxy2.id)
        self.assertEqual(self.ndpproxy2, np2)
        router1_ndp_proxies = self.mapping.get_ndp_proxies_by_router_id(
            self.router1)
        self.assertEqual([], router1_ndp_proxies)
        router2_ndp_proxies = self.mapping.get_ndp_proxies_by_router_id(
            self.router2)
        self.assertEqual([self.ndpproxy2], router2_ndp_proxies)



# Copyright 2023 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_log import log
from ovsdbapp.backend.ovs_idl import connection
from ovsdbapp.backend.ovs_idl import idlutils
from ovsdbapp.schema.open_vswitch import impl_idl as impl_idl_ovs

from neutron.agent.ovsdb.native import connection as ovsdb_conn
from neutron.common.ovn import constants as ovn_const
from neutron.common.ovn import utils as ovn_utils
from neutron.conf.plugins.ml2.drivers.ovn import ovn_conf as config
from neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb import impl_idl_ovn
from neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb import ovsdb_monitor


LOG = log.getLogger(__name__)


class MonitorAgentOvnSbIdl(ovsdb_monitor.OvnIdl):

    SCHEMA = 'OVN_Southbound'

    def __init__(self, tables, events, chassis=None):
        connection_string = config.get_ovn_sb_connection()
        ovsdb_monitor._check_and_set_ssl_files(self.SCHEMA)
        helper = self._get_ovsdb_helper(connection_string)
        for table in tables:
            helper.register_table(table)
        try:
            super().__init__(None, connection_string, helper,
                             leader_only=False)
        except TypeError:
            # TODO(twilson) We can remove this when we require ovs>=2.12.0
            super().__init__(None, connection_string, helper)
        if chassis:
            for table in set(tables).intersection({'Chassis',
                                                   'Chassis_Private'}):
                self.set_table_condition(table, [['name', '==', chassis]])
        if events:
            self.notify_handler.watch_events(events)

    @ovn_utils.retry(max_=180)
    def _get_ovsdb_helper(self, connection_string):
        return idlutils.get_schema_helper(connection_string, self.SCHEMA)

    @ovn_utils.retry()
    def start(self):
        LOG.info('Getting OvsdbSbOvnIdl for OVN monitor with retry')
        conn = connection.Connection(
            self, timeout=config.get_ovn_ovsdb_timeout())
        return impl_idl_ovn.OvsdbSbOvnIdl(conn)

    def post_connect(self):
        pass


class MonitorAgentOvnNbIdl(ovsdb_monitor.OvnIdl):

    SCHEMA = 'OVN_Northbound'

    def __init__(self, tables, events):
        connection_string = config.get_ovn_nb_connection()
        ovsdb_monitor._check_and_set_ssl_files(self.SCHEMA)
        helper = self._get_ovsdb_helper(connection_string)
        for table in tables:
            helper.register_table(table)
        try:
            super().__init__(None, connection_string, helper,
                             leader_only=False)
        except TypeError:
            # TODO(twilson) We can remove this when we require ovs>=2.12.0
            super().__init__(None, connection_string, helper)
        if events:
            self.notify_handler.watch_events(events)

    @ovn_utils.retry(max_=180)
    def _get_ovsdb_helper(self, connection_string):
        return idlutils.get_schema_helper(connection_string, self.SCHEMA)

    @ovn_utils.retry()
    def start(self):
        LOG.info('Getting OvsdbNbOvnIdl for OVN monitor with retry')
        conn = connection.Connection(
            self, timeout=config.get_ovn_ovsdb_timeout())
        return impl_idl_ovn.OvsdbNbOvnIdl(conn)

    def post_connect(self):
        pass


class MonitorAgentOvsIdl(ovsdb_conn.OvsIdl):

    def __init__(self, events):
        super().__init__()
        if events:
            self.notify_handler.watch_events(events)

    @ovn_utils.retry()
    def start(self):
        LOG.info('Getting OvsdbIdl for OVN monitor with retry')
        conn = connection.Connection(self,
                                     timeout=config.get_ovn_ovsdb_timeout())
        return impl_idl_ovs.OvsdbIdl(conn)

    def post_connect(self):
        pass


def get_ovn_bridge(ovs_idl):
    """Return the external_ids:ovn-bridge value of the Open_vSwitch table.

    This is the OVS bridge used to plug the metadata ports to.
    If the key doesn't exist, this method will return 'br-int' as default.
    """
    ext_ids = ovs_idl.db_get('Open_vSwitch', '.', 'external_ids').execute()
    try:
        return ext_ids['ovn-bridge']
    except KeyError:
        LOG.warning("Can't read ovn-bridge external-id from OVSDB. Using "
                    "br-int instead.")
        return 'br-int'


def get_own_chassis_name(ovs_idl):
    """Return the external_ids:system-id value of the Open_vSwitch table.

    As long as ovn-controller is running on this node, the key is
    guaranteed to exist and will include the chassis name.
    """
    ext_ids = ovs_idl.db_get('Open_vSwitch', '.', 'external_ids').execute()
    return ext_ids['system-id']


def get_ovs_port_name(ovs_idl, port_id):
    """Return the OVS port name given the Neutron port ID"""
    int_list = ovs_idl.db_list('Interface', columns=['name', 'external_ids'],
                               if_exists=True).execute(check_error=True,
                                                       log_errors=False)
    for interface in int_list:
        if interface['external_ids'].get('iface-id') == port_id:
            return interface['name']


def get_port_qos(nb_idl, port_id):
    """Retrieve the QoS egress max-bw and min-bw values (in kbps) of a LSP

    There could be max-bw rules ingress (to-lport) and egress (from-lport);
    this method is only returning the egress one. The min-bw rule is only
    implemented for egress traffic.
    """
    try:
        lsp = nb_idl.lsp_get(port_id).execute(check_error=True)
    except idlutils.RowNotFound:
        # If the LSP is not present, we can't retrieve any QoS info. The
        # default values are (0, 0).
        return 0, 0

    net_name = lsp.external_ids[ovn_const.OVN_NETWORK_NAME_EXT_ID_KEY]
    ls = nb_idl.lookup('Logical_Switch', net_name)
    for qos_rule in iter(r for r in ls.qos_rules if
                         r.external_ids[ovn_const.OVN_PORT_EXT_ID_KEY]):
        if qos_rule.direction != 'from-lport':
            continue

        max_kbps = int(qos_rule.bandwidth.get('rate', 0))
        break
    else:
        max_kbps = 0

    min_kbps = int(lsp.options.get(ovn_const.LSP_OPTIONS_QOS_MIN_RATE, 0))
    return max_kbps, min_kbps



# Copyright (c) 2021 China Unicom Cloud Data Co.,Ltd.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from neutron_lib import context
from oslo_config import cfg

from neutron.agent.common import ovs_lib
from neutron.agent.l2.extensions.dhcp import extension as ovs_dhcp
from neutron.plugins.ml2.drivers.openvswitch.agent \
    import ovs_agent_extension_api as ovs_ext_api
from neutron.tests import base


class DHCPAgentExtensionTestCase(base.BaseTestCase):

    def setUp(self):
        super(DHCPAgentExtensionTestCase, self).setUp()
        cfg.CONF.set_override('enable_ipv6', True, group='DHCP')
        self.context = context.get_admin_context()
        self.int_br = mock.Mock()
        self.tun_br = mock.Mock()
        self.plugin_rpc = mock.Mock()
        self.remote_resource_cache = mock.Mock()
        self.plugin_rpc.remote_resource_cache = self.remote_resource_cache
        self.ovs_dhcp = ovs_dhcp.DHCPAgentExtension()
        self.agent_api = ovs_ext_api.OVSAgentExtensionAPI(
            self.int_br,
            self.tun_br,
            phys_brs=None,
            plugin_rpc=self.plugin_rpc)
        self.ovs_dhcp.consume_api(self.agent_api)
        self.ovs_dhcp.initialize(None, None)

    def tearDown(self):
        self.ovs_dhcp.app_mgr.uninstantiate(self.ovs_dhcp.dhcp4_app.name)
        self.ovs_dhcp.app_mgr.uninstantiate(self.ovs_dhcp.dhcp6_app.name)
        super(DHCPAgentExtensionTestCase, self).tearDown()

    def test_handle_port(self):
        port = {"port_id": "p1",
                "fixed_ips": [{"ip_address": "1.1.1.1"}],
                "vif_port": ovs_lib.VifPort("tap-p1", "1", "p1",
                                            "aa:aa:aa:aa:aa:aa", "br-int"),
                "device_owner": "compute:test"}
        self.ovs_dhcp.handle_port(self.context, port)
        self.ovs_dhcp.int_br.add_dhcp_ipv4_flow.assert_called_once_with(
            port['port_id'],
            port["vif_port"].ofport,
            port["vif_port"].vif_mac)
        self.ovs_dhcp.int_br.add_dhcp_ipv6_flow.assert_called_once_with(
            port['port_id'],
            port["vif_port"].ofport,
            port["vif_port"].vif_mac)
        self.assertIsNotNone(self.ovs_dhcp.VIF_PORT_CACHE.get(port['port_id']))

    def _test_delete_port(self, with_vif_port=False):
        vif_port = ovs_lib.VifPort("tap-p1", 1, "p1",
                                   "aa:aa:aa:aa:aa:aa", "br-int")
        port1 = {"port_id": "p1",
                 "fixed_ips": [{"ip_address": "1.1.1.1"}],
                 "vif_port": vif_port,
                 "device_owner": "compute:test"}
        self.ovs_dhcp.handle_port(self.context, port1)
        with mock.patch.object(self.ovs_dhcp.int_br, "get_vif_ports",
                               return_value=[]):
            if with_vif_port:
                port2 = {"port_id": "p1",
                         "vif_port": vif_port}
            else:
                port2 = {"port_id": "p1"}
            self.ovs_dhcp.delete_port(self.context, port2)
            self.ovs_dhcp.int_br.del_dhcp_flow.assert_called_once_with(
                port1["vif_port"].ofport,
                port1["vif_port"].vif_mac)
            # verify the cache
            self.assertNotIn("p1", self.ovs_dhcp.VIF_PORT_CACHE.keys())

    def test_delete_port_without_vif_port(self):
        self._test_delete_port()

    def test_delete_port_with_vif_port(self):
        self._test_delete_port(with_vif_port=True)



# Copyright (c) 2021 China Unicom Cloud Data Co.,Ltd.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

"""add qos policy rule Packet Rate Limit

Revision ID: 1bb3393de75d
Revises: 8df53b0d2c0e
Create Date: 2021-01-22 17:00:03.085196

"""

from alembic import op
import sqlalchemy as sa

from neutron_lib import constants

from neutron.db import migration

# revision identifiers, used by Alembic.
revision = '1bb3393de75d'
down_revision = '8df53b0d2c0e'

# milestone identifier, used by neutron-db-manage
neutron_milestone = [migration.XENA]

direction_enum = sa.Enum(
    constants.EGRESS_DIRECTION, constants.INGRESS_DIRECTION,
    name='qos_packet_rate_limit_rules_directions'
)


def upgrade():
    op.create_table(
        'qos_packet_rate_limit_rules',
        sa.Column('id', sa.String(36), primary_key=True),
        sa.Column('qos_policy_id',
                  sa.String(length=36),
                  sa.ForeignKey('qos_policies.id', ondelete='CASCADE'),
                  nullable=False, index=True),
        sa.Column('max_kpps', sa.Integer()),
        sa.Column('max_burst_kpps', sa.Integer()),
        sa.Column('direction', direction_enum,
                  nullable=False,
                  server_default=constants.EGRESS_DIRECTION),
        sa.UniqueConstraint('qos_policy_id', 'direction',
            name='qos_packet_rate_limit_rules0qos_policy_id0direction')
    )



# Copyright 2016 Red Hat, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import functools
import random

import netaddr

from neutron.tests.common.exclusive_resources import resource_allocator

TEST_NET_RANGE = {
    1: ('192.0.2.1', '192.0.2.254'),
    2: ('198.51.100.1', '198.51.100.254'),
    3: ('203.0.113.1', '203.0.113.254'),
}


def get_test_net_address_fixture(test_net_number):
    """Return exclusive ip address on the system based on RFC 5737.

    :param block: One of following constants: 1, 2, 3

    https://tools.ietf.org/html/rfc5737
    """
    try:
        net_range = TEST_NET_RANGE[test_net_number]
    except KeyError:
        raise ValueError("Unknown constant for TEST-NET: %d" % test_net_number)

    return ExclusiveIPAddress(*net_range)


def get_random_ip(low, high):
    parent_range = netaddr.IPRange(low, high)
    return str(random.choice(parent_range))


class ExclusiveIPAddress(resource_allocator.ExclusiveResource):
    """Allocate a unique ip address.

    :ivar address: allocated ip address
    :type address: netaddr.IPAddress
    """

    def __init__(self, low, high):
        super(ExclusiveIPAddress, self).__init__(
            'ip_addresses', functools.partial(get_random_ip, low, high))

    def _setUp(self):
        super(ExclusiveIPAddress, self)._setUp()
        self.address = netaddr.IPAddress(self.resource)



# Copyright 2016 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import abc

from neutron_lib import rpc as n_rpc
from oslo_log import helpers as log_helpers
import oslo_messaging

from neutron.api.rpc.callbacks.consumer import registry
from neutron.api.rpc.callbacks import resources
from neutron.api.rpc.handlers import resources_rpc
from neutron.services.trunk.rpc import constants as trunk_consts

# This module contains stub (client-side) and skeleton (server-side)
# proxy code that executes in the Neutron L2 Agent process space. This
# is needed if trunk service plugin drivers have a remote component
# (e.g. agent), that needs to communicate with the Neutron Server.

# The Agent side exposes the following remote methods:
#
# - update methods to learn about a trunk and its subports: these
#   methods are used by the server to tell the agent about trunk
#   updates; agents may selectively choose to listen to either
#   trunk or subports updates or both.
#
# For server-side stub and skeleton proxy code, please look at server.py


class TrunkSkeleton(object):
    """Skeleton proxy code for server->agent communication."""

    def __init__(self):
        registry.register(self.handle_trunks, resources.TRUNK)
        registry.register(self.handle_subports, resources.SUBPORT)

        self._connection = n_rpc.Connection()
        endpoints = [resources_rpc.ResourcesPushRpcCallback()]
        topic = resources_rpc.resource_type_versioned_topic(resources.SUBPORT)
        self._connection.create_consumer(topic, endpoints, fanout=True)
        topic = resources_rpc.resource_type_versioned_topic(resources.TRUNK)
        self._connection.create_consumer(topic, endpoints, fanout=True)
        self._connection.consume_in_threads()

    def unregister(self):
        self._connection.close()

    @abc.abstractmethod
    def handle_trunks(self, context, resource_type, trunks, event_type):
        """Handle trunk events."""
        # if common logic may be extracted out, consider making a base
        # version of this method that can be overridden by the inherited
        # skeleton.
        # NOTE: If trunk is not managed by the agent, the notification can
        # either be ignored or cached for future use.

    @abc.abstractmethod
    def handle_subports(self, context, resource_type, subports, event_type):
        """Handle subports event."""
        # if common logic may be extracted out, consider making a base
        # version of this method that can be overridden by the inherited
        # skeleton.
        # NOTE: If the subport belongs to a trunk which the agent does not
        # manage, the notification should be ignored.


class TrunkStub(object):
    """Stub proxy code for agent->server communication."""
    # API HISTORY
    #   1.0 - initial version
    VERSION = '1.0'

    def __init__(self):
        self.stub = resources_rpc.ResourcesPullRpcApi()
        target = oslo_messaging.Target(
            topic=trunk_consts.TRUNK_BASE_TOPIC,
            version=self.VERSION,
            namespace=trunk_consts.TRUNK_BASE_NAMESPACE)
        self.rpc_client = n_rpc.get_client(target)

    @log_helpers.log_method_call
    def get_trunk_details(self, context, parent_port_id):
        """Get information about the trunk for the given parent port."""
        return self.stub.pull(context, resources.TRUNK, parent_port_id)

    @log_helpers.log_method_call
    def update_trunk_status(self, context, trunk_id, status):
        """Update the trunk status to reflect outcome of data plane wiring."""
        return self.rpc_client.prepare().call(
            context, 'update_trunk_status',
            trunk_id=trunk_id, status=status)

    @log_helpers.log_method_call
    def update_subport_bindings(self, context, subports):
        """Update subport bindings to match parent port host binding."""
        return self.rpc_client.prepare().call(
            context, 'update_subport_bindings', subports=subports)



# Copyright (c) 2015 Red Hat, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib.plugins import constants
from neutron_lib.plugins import directory

from neutron.tests.unit.plugins.ml2 import test_plugin


class ML2TestFramework(test_plugin.Ml2PluginV2TestCase):
    l3_plugin = ('neutron.services.l3_router.l3_router_plugin.'
                 'L3RouterPlugin')
    _mechanism_drivers = ['openvswitch']

    def get_additional_service_plugins(self):
        p = super(ML2TestFramework, self).get_additional_service_plugins()
        p.update({'flavors_plugin_name': 'neutron.services.flavors.'
                                         'flavors_plugin.FlavorsPlugin'})
        return p

    def setUp(self):
        super(ML2TestFramework, self).setUp()
        self.core_plugin = directory.get_plugin()
        self.l3_plugin = directory.get_plugin(constants.L3)

    def _create_router(self, distributed=False, ha=False, admin_state_up=True):
        return self.l3_plugin.create_router(
            self.context,
            {'router':
             {'name': 'router',
              'admin_state_up': admin_state_up,
              'tenant_id': self._tenant_id,
              'ha': ha,
              'distributed': distributed}})



# Copyright (c) 2021 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_policy import policy as base_policy

from neutron import policy
from neutron.tests.unit.conf.policies import test_base as base


class NetworkAPITestCase(base.PolicyBaseTestCase):

    def setUp(self):
        super(NetworkAPITestCase, self).setUp()
        self.target = {'project_id': self.project_id}
        self.alt_target = {'project_id': self.alt_project_id}


class SystemAdminTests(NetworkAPITestCase):

    def setUp(self):
        super(SystemAdminTests, self).setUp()
        self.context = self.system_admin_ctx

    def test_create_network(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce, self.context, 'create_network', self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce, self.context, 'create_network', self.alt_target)

    def test_create_network_shared(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'create_network:shared', self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'create_network:shared', self.alt_target)

    def test_create_network_external(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'create_network:router:external', self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'create_network:router:external', self.alt_target)

    def test_create_network_default(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'create_network:is_default', self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'create_network:is_default', self.alt_target)

    def test_create_network_port_security_enabled(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'create_network:port_security_enabled',
            self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'create_network:port_security_enabled',
            self.alt_target)

    def test_create_network_segments(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'create_network:segments', self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'create_network:segments', self.alt_target)

    def test_create_network_provider_network_type(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'create_network:provider:network_type', self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'create_network:provider:network_type',
            self.alt_target)

    def test_create_network_provider_physical_network(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'create_network:provider:physical_network',
            self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'create_network:provider:physical_network',
            self.alt_target)

    def test_create_network_provider_segmentation_id(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'create_network:provider:segmentation_id',
            self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'create_network:provider:segmentation_id',
            self.alt_target)

    def test_get_network(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'get_network',
            self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'get_network',
            self.alt_target)

    def test_get_network_segments(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'get_network:segments', self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'get_network:segments', self.alt_target)

    def test_get_network_provider_network_type(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'get_network:provider:network_type', self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'get_network:provider:network_type', self.alt_target)

    def test_get_network_provider_physical_network(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'get_network:provider:physical_network',
            self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'get_network:provider:physical_network',
            self.alt_target)

    def test_get_network_provider_segmentation_id(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'get_network:provider:segmentation_id',
            self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'get_network:provider:segmentation_id',
            self.alt_target)

    def test_get_networks_tags(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'get_networks_tags',
            self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'get_networks_tags',
            self.alt_target)

    def test_update_network(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce, self.context, 'update_network', self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce, self.context, 'update_network', self.alt_target)

    def test_update_network_segments(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'update_network:segments', self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'update_network:segments', self.alt_target)

    def test_update_network_shared(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'update_network:shared', self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'update_network:shared', self.alt_target)

    def test_update_network_provider_network_type(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'update_network:provider:network_type', self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'update_network:provider:network_type',
            self.alt_target)

    def test_update_network_provider_physical_network(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'update_network:provider:physical_network',
            self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'update_network:provider:physical_network',
            self.alt_target)

    def test_update_network_provider_segmentation_id(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'update_network:provider:segmentation_id',
            self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'update_network:provider:segmentation_id',
            self.alt_target)

    def test_update_network_external(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'update_network:router:external', self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'update_network:router:external', self.alt_target)

    def test_update_network_default(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'update_network:is_default', self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'update_network:is_default', self.alt_target)

    def test_update_network_port_security_enabled(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'update_network:port_security_enabled',
            self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'update_network:port_security_enabled',
            self.alt_target)

    def test_update_networks_tags(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce, self.context, 'update_networks_tags', self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce, self.context, 'update_networks_tags',
            self.alt_target)

    def test_delete_network(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce, self.context, 'delete_network', self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce, self.context, 'delete_network', self.alt_target)

    def test_delete_networks_tags(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce, self.context, 'delete_networks_tags', self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce, self.context, 'delete_networks_tags',
            self.alt_target)


class SystemMemberTests(SystemAdminTests):

    def setUp(self):
        super(SystemMemberTests, self).setUp()
        self.context = self.system_member_ctx


class SystemReaderTests(SystemMemberTests):

    def setUp(self):
        super(SystemReaderTests, self).setUp()
        self.context = self.system_reader_ctx


class AdminTests(NetworkAPITestCase):

    def setUp(self):
        super(AdminTests, self).setUp()
        self.context = self.project_admin_ctx

    def test_create_network(self):
        self.assertTrue(
            policy.enforce(self.context, 'create_network', self.target))
        self.assertTrue(
            policy.enforce(self.context, 'create_network', self.alt_target))

    def test_create_network_shared(self):
        self.assertTrue(
            policy.enforce(self.context, 'create_network:shared', self.target))
        self.assertTrue(
            policy.enforce(
                self.context, 'create_network:shared', self.alt_target))

    def test_create_network_external(self):
        self.assertTrue(
            policy.enforce(self.context,
                           'create_network:router:external', self.target))
        self.assertTrue(
            policy.enforce(self.context,
                           'create_network:router:external', self.alt_target))

    def test_create_network_default(self):
        self.assertTrue(
            policy.enforce(self.context,
                           'create_network:is_default', self.target))
        self.assertTrue(
            policy.enforce(self.context,
                           'create_network:is_default', self.alt_target))

    def test_create_network_port_security_enabled(self):
        self.assertTrue(
            policy.enforce(self.context,
                           'create_network:port_security_enabled',
                           self.target))
        self.assertTrue(
            policy.enforce(self.context,
                           'create_network:port_security_enabled',
                           self.alt_target))

    def test_create_network_segments(self):
        self.assertTrue(
            policy.enforce(self.context,
                           'create_network:segments', self.target))
        self.assertTrue(
            policy.enforce(self.context,
                           'create_network:segments', self.alt_target))

    def test_create_network_provider_network_type(self):
        self.assertTrue(
            policy.enforce(self.context,
                           'create_network:provider:network_type',
                           self.target))
        self.assertTrue(
            policy.enforce(self.context,
                           'create_network:provider:network_type',
                           self.alt_target))

    def test_create_network_provider_physical_network(self):
        self.assertTrue(
            policy.enforce(self.context,
                           'create_network:provider:physical_network',
                           self.target))
        self.assertTrue(
            policy.enforce(self.context,
                           'create_network:provider:physical_network',
                           self.alt_target))

    def test_create_network_provider_segmentation_id(self):
        self.assertTrue(
            policy.enforce(self.context,
                           'create_network:provider:segmentation_id',
                           self.target))
        self.assertTrue(
            policy.enforce(self.context,
                           'create_network:provider:segmentation_id',
                           self.alt_target))

    def test_get_network(self):
        self.assertTrue(
            policy.enforce(self.context, 'get_network', self.target))
        self.assertTrue(
            policy.enforce(self.context, 'get_network', self.alt_target))

    def test_get_network_provider_network_type(self):
        self.assertTrue(
            policy.enforce(self.context,
                           'get_network:provider:network_type',
                           self.target))
        self.assertTrue(
            policy.enforce(self.context,
                           'get_network:provider:network_type',
                           self.alt_target))

    def test_get_network_provider_physical_network(self):
        self.assertTrue(
            policy.enforce(self.context,
                           'get_network:provider:physical_network',
                           self.target))
        self.assertTrue(
            policy.enforce(self.context,
                           'get_network:provider:physical_network',
                           self.alt_target))

    def test_get_network_provider_segmentation_id(self):
        self.assertTrue(
            policy.enforce(self.context,
                           'get_network:provider:segmentation_id',
                           self.target))
        self.assertTrue(
            policy.enforce(self.context,
                           'get_network:provider:segmentation_id',
                           self.alt_target))

    def test_get_networks_tags(self):
        self.assertTrue(
            policy.enforce(self.context, 'get_networks_tags', self.target))
        self.assertTrue(
            policy.enforce(self.context, 'get_networks_tags', self.alt_target))

    def test_update_network(self):
        self.assertTrue(
            policy.enforce(self.context, 'update_network', self.target))
        self.assertTrue(
            policy.enforce(self.context, 'update_network', self.alt_target))

    def test_update_network_segments(self):
        self.assertTrue(
            policy.enforce(self.context,
                           'update_network:segments', self.target))
        self.assertTrue(
            policy.enforce(self.context,
                           'update_network:segments', self.alt_target))

    def test_update_network_shared(self):
        self.assertTrue(
            policy.enforce(self.context, 'update_network:shared', self.target))
        self.assertTrue(
            policy.enforce(
                self.context, 'update_network:shared', self.alt_target))

    def test_update_network_provider_network_type(self):
        self.assertTrue(
            policy.enforce(self.context,
                           'update_network:provider:network_type',
                           self.target))
        self.assertTrue(
            policy.enforce(self.context,
                           'update_network:provider:network_type',
                           self.alt_target))

    def test_update_network_provider_physical_network(self):
        self.assertTrue(
            policy.enforce(self.context,
                           'update_network:provider:physical_network',
                           self.target))
        self.assertTrue(
            policy.enforce(self.context,
                           'update_network:provider:physical_network',
                           self.alt_target))

    def test_update_network_provider_segmentation_id(self):
        self.assertTrue(
            policy.enforce(self.context,
                           'update_network:provider:segmentation_id',
                           self.target))
        self.assertTrue(
            policy.enforce(self.context,
                           'update_network:provider:segmentation_id',
                           self.alt_target))

    def test_update_network_external(self):
        self.assertTrue(
            policy.enforce(self.context,
                           'update_network:router:external', self.target))
        self.assertTrue(
            policy.enforce(self.context,
                           'update_network:router:external', self.alt_target))

    def test_update_network_default(self):
        self.assertTrue(
            policy.enforce(self.context,
                           'update_network:is_default', self.target))
        self.assertTrue(
            policy.enforce(self.context,
                           'update_network:is_default', self.alt_target))

    def test_update_network_port_security_enabled(self):
        self.assertTrue(
            policy.enforce(self.context,
                           'update_network:port_security_enabled',
                           self.target))
        self.assertTrue(
            policy.enforce(self.context,
                           'update_network:port_security_enabled',
                           self.alt_target))

    def test_update_networks_tags(self):
        self.assertTrue(
            policy.enforce(self.context, 'update_networks_tags', self.target))
        self.assertTrue(
            policy.enforce(self.context, 'update_networks_tags',
                           self.alt_target))

    def test_delete_network(self):
        self.assertTrue(
            policy.enforce(self.context, 'delete_network', self.target))
        self.assertTrue(
            policy.enforce(self.context, 'delete_network', self.alt_target))

    def test_delete_networks_tags(self):
        self.assertTrue(
            policy.enforce(self.context, 'delete_networks_tags', self.target))
        self.assertTrue(
            policy.enforce(self.context, 'delete_networks_tags',
                           self.alt_target))


class ProjectMemberTests(AdminTests):

    def setUp(self):
        super(ProjectMemberTests, self).setUp()
        self.context = self.project_member_ctx

    def test_create_network(self):
        self.assertTrue(
            policy.enforce(self.context, 'create_network', self.target))
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'create_network', self.alt_target)

    def test_create_network_shared(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'create_network:shared', self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'create_network:shared', self.alt_target)

    def test_create_network_external(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'create_network:router:external', self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'create_network:router:external', self.alt_target)

    def test_create_network_default(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'create_network:is_default', self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'create_network:is_default', self.alt_target)

    def test_create_network_port_security_enabled(self):
        self.assertTrue(
            policy.enforce(self.context,
                           'create_network:port_security_enabled',
                           self.target))
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'create_network:port_security_enabled',
            self.alt_target)

    def test_create_network_segments(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'create_network:segments', self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'create_network:segments', self.alt_target)

    def test_create_network_provider_network_type(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'create_network:provider:network_type', self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'create_network:provider:network_type',
            self.alt_target)

    def test_create_network_provider_physical_network(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'create_network:provider:physical_network',
            self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'create_network:provider:physical_network',
            self.alt_target)

    def test_create_network_provider_segmentation_id(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'create_network:provider:segmentation_id',
            self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'create_network:provider:segmentation_id',
            self.alt_target)

    def test_get_network(self):
        self.assertTrue(
            policy.enforce(self.context, 'get_network', self.target))
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'get_network', self.alt_target)

    def test_get_network_segments(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'get_network:segments', self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'get_network:segments', self.alt_target)

    def test_get_network_provider_network_type(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'get_network:provider:network_type', self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'get_network:provider:network_type', self.alt_target)

    def test_get_network_provider_physical_network(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'get_network:provider:physical_network',
            self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'get_network:provider:physical_network',
            self.alt_target)

    def test_get_network_provider_segmentation_id(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'get_network:provider:segmentation_id',
            self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'get_network:provider:segmentation_id',
            self.alt_target)

    def test_get_networks_tags(self):
        self.assertTrue(
            policy.enforce(self.context, 'get_networks_tags', self.target))
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'get_networks_tags', self.alt_target)

    def test_update_network(self):
        self.assertTrue(
            policy.enforce(self.context, 'update_network', self.target))
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_network', self.alt_target)

    def test_update_network_segments(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_network:segments', self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_network:segments', self.alt_target)

    def test_update_network_shared(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_network:shared', self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_network:shared', self.alt_target)

    def test_update_network_provider_network_type(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_network:provider:network_type', self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_network:provider:network_type',
            self.alt_target)

    def test_update_network_provider_physical_network(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_network:provider:physical_network',
            self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_network:provider:physical_network',
            self.alt_target)

    def test_update_network_provider_segmentation_id(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_network:provider:segmentation_id',
            self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_network:provider:segmentation_id',
            self.alt_target)

    def test_update_network_external(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_network:router:external', self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_network:router:external', self.alt_target)

    def test_update_network_default(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_network:is_default', self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_network:is_default', self.alt_target)

    def test_update_network_port_security_enabled(self):
        self.assertTrue(
            policy.enforce(self.context,
                           'update_network:port_security_enabled',
                           self.target))
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_network:port_security_enabled',
            self.alt_target)

    def test_update_networks_tags(self):
        self.assertTrue(
            policy.enforce(self.context, 'update_networks_tags', self.target))
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_networks_tags', self.alt_target)

    def test_delete_network(self):
        self.assertTrue(
            policy.enforce(self.context, 'delete_network', self.target))
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'delete_network', self.alt_target)

    def test_delete_networks_tags(self):
        self.assertTrue(
            policy.enforce(self.context, 'delete_networks_tags', self.target))
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'delete_networks_tags', self.alt_target)


class ProjectReaderTests(ProjectMemberTests):

    def setUp(self):
        super(ProjectReaderTests, self).setUp()
        self.context = self.project_reader_ctx

    def test_create_network(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce, self.context, 'create_network', self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce, self.context, 'create_network', self.alt_target)

    def test_create_network_port_security_enabled(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'create_network:port_security_enabled',
            self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'create_network:port_security_enabled',
            self.alt_target)

    def test_update_network(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce, self.context, 'update_network', self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce, self.context, 'update_network', self.alt_target)

    def test_update_network_port_security_enabled(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_network:port_security_enabled',
            self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_network:port_security_enabled',
            self.alt_target)

    def test_update_networks_tags(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce, self.context, 'update_networks_tags', self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce, self.context, 'update_networks_tags',
            self.alt_target)

    def test_delete_network(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce, self.context, 'delete_network', self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce, self.context, 'delete_network', self.alt_target)

    def test_delete_networks_tags(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce, self.context, 'delete_networks_tags', self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce, self.context, 'delete_networks_tags',
            self.alt_target)


class ServiceRoleTests(NetworkAPITestCase):

    def setUp(self):
        super(ServiceRoleTests, self).setUp()
        self.context = self.service_ctx

    def test_create_network(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce, self.context, 'create_network', self.target)

    def test_create_network_shared(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'create_network:shared', self.target)

    def test_create_network_external(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'create_network:router:external', self.target)

    def test_create_network_default(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'create_network:is_default', self.target)

    def test_create_network_port_security_enabled(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'create_network:port_security_enabled',
            self.target)

    def test_create_network_segments(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'create_network:segments', self.target)

    def test_create_network_provider_network_type(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'create_network:provider:network_type', self.target)

    def test_create_network_provider_physical_network(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'create_network:provider:physical_network',
            self.target)

    def test_create_network_provider_segmentation_id(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'create_network:provider:segmentation_id',
            self.target)

    def test_get_network(self):
        self.assertTrue(
            policy.enforce(self.context, 'get_network', self.target))

    def test_get_network_segments(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'get_network:segments', self.target)

    def test_get_network_provider_network_type(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'get_network:provider:network_type', self.target)

    def test_get_network_provider_physical_network(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'get_network:provider:physical_network',
            self.target)

    def test_get_network_provider_segmentation_id(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'get_network:provider:segmentation_id',
            self.target)

    def test_update_network(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce, self.context, 'update_network', self.target)

    def test_update_network_segments(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_network:segments', self.target)

    def test_update_network_shared(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_network:shared', self.target)

    def test_update_network_provider_network_type(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_network:provider:network_type', self.target)

    def test_update_network_provider_physical_network(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_network:provider:physical_network',
            self.target)

    def test_update_network_provider_segmentation_id(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_network:provider:segmentation_id',
            self.target)

    def test_update_network_external(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_network:router:external', self.target)

    def test_update_network_default(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_network:is_default', self.target)

    def test_update_network_port_security_enabled(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_network:port_security_enabled',
            self.target)

    def test_delete_network(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce, self.context, 'delete_network', self.target)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from neutron_lib import constants
from neutron_lib import exceptions

from neutron.agent.linux import l3_tc_lib as tc_lib
from neutron.agent.linux import tc_lib as base_tc_lib
from neutron.tests import base

FLOATING_IP_DEVICE_NAME = "qg-device_rfp"
FLOATING_IP_ROUTER_NAMESPACE = "qrouter-namespace_snat-namespace"

FLOATING_IP_1 = "172.16.5.146"
FLOATING_IP_2 = "172.16.10.105"
FILETER_ID_1 = "800::800"
FILETER_ID_2 = "800::801"

TC_INGRESS_FILTERS_BASE = (
    'filter protocol ip u32 \n'
    'filter protocol ip u32 %(chain_value)sfh 800: ht divisor 1 \n'
    'filter protocol ip u32 %(chain_value)sfh %(filter_id1)s order 2048 key '
    'ht 800 bkt 0 '
    'flowid :1  (rule hit 0 success 0)\n'
    '  match IP dst %(fip1)s/32 (success 0 ) \n'
    ' police 0x3 rate 3000Kbit burst 3Mb mtu 64Kb action drop overhead 0b \n'
    'ref 1 bind 1\n'
    '\n'
    ' Sent 111 bytes 222 pkts (dropped 0, overlimits 0) \n'
    'filter protocol ip u32 %(chain_value)sfh %(filter_id2)s order 2049 key '
    'ht 800 bkt 0 '
    'flowid :1  (rule hit 0 success 0)\n'
    '  match IP dst %(fip2)s/32 (success 0 ) \n'
    ' police 0x1b rate 22000Kbit burst 22Mb mtu 64Kb action drop '
    'overhead 0b \n'
    'ref 1 bind 1\n'
    '\n'
    ' Sent 111 bytes 222 pkts (dropped 0, overlimits 0)\n')

TC_INGRESS_FILTERS_WITHOUT_CHAIN = TC_INGRESS_FILTERS_BASE % {
    "chain_value": "",
    "filter_id1": FILETER_ID_1,
    "fip1": FLOATING_IP_1,
    "filter_id2": FILETER_ID_2,
    "fip2": FLOATING_IP_2}

# NOTE(slaweq): in iproute 4.15 chain value was added to filter output
TC_INGRESS_FILTERS_WITH_CHAIN = TC_INGRESS_FILTERS_BASE % {
    "chain_value": "chain 1 ",
    "filter_id1": FILETER_ID_1,
    "fip1": FLOATING_IP_1,
    "filter_id2": FILETER_ID_2,
    "fip2": FLOATING_IP_2}

TC_INGRESS_FILTERS_DUP_WITHOUT_CHAIN = TC_INGRESS_FILTERS_WITHOUT_CHAIN + (
    'filter protocol ip u32 %(chain_value)sfh %(filter_id2)s order 2049 key '
    'ht 800 bkt 0 '
    'flowid :1  (rule hit 0 success 0)\n'
    '  match IP dst %(fip2)s/32 (success 0 ) \n'
    ' police 0x1b rate 22000Kbit burst 22Mb mtu 64Kb action drop '
    'overhead 0b \n'
    'ref 1 bind 1\n'
    '\n'
    ' Sent 111 bytes 222 pkts (dropped 0, overlimits 0)\n') % {
        "chain_value": "",
        "filter_id2": FILETER_ID_2,
        "fip2": FLOATING_IP_2}

TC_INGRESS_FILTERS_DUP_WITH_CHAIN = TC_INGRESS_FILTERS_WITH_CHAIN + (
    'filter protocol ip u32 %(chain_value)sfh %(filter_id2)s order 2049 key '
    'ht 800 bkt 0 '
    'flowid :1  (rule hit 0 success 0)\n'
    '  match IP dst %(fip2)s/32 (success 0 ) \n'
    ' police 0x1b rate 22000Kbit burst 22Mb mtu 64Kb action drop '
    'overhead 0b \n'
    'ref 1 bind 1\n'
    '\n'
    ' Sent 111 bytes 222 pkts (dropped 0, overlimits 0)\n') % {
        "chain_value": "chain 1 ",
        "filter_id2": FILETER_ID_2,
        "fip2": FLOATING_IP_2}

TC_EGRESS_FILTERS_BASE = (
    'filter protocol ip u32 \n'
    'filter protocol ip u32 %(chain_name)sfh 800: ht divisor 1 \n'
    'filter protocol ip u32 %(chain_name)sfh %(filter_id1)s order 2048 key '
    'ht 800 bkt 0 '
    'flowid :1  (rule hit 0 success 0)\n'
    '  match IP src %(fip1)s/32 (success 0 ) \n'
    ' police 0x4 rate 3000Kbit burst 3Mb mtu 64Kb action drop overhead 0b \n'
    'ref 1 bind 1\n'
    '\n'
    ' Sent 111 bytes 222 pkts (dropped 0, overlimits 0) \n'
    'filter protocol ip u32 %(chain_name)sfh %(filter_id2)s order 2049 key '
    'ht 800 bkt 0 '
    'flowid :1  (rule hit 0 success 0)\n'
    '  match IP src %(fip2)s/32 (success 0 ) \n'
    ' police 0x1c rate 22000Kbit burst 22Mb mtu 64Kb action drop '
    'overhead 0b \n'
    'ref 1 bind 1\n'
    '\n'
    ' Sent 111 bytes 222 pkts (dropped 0, overlimits 0)\n')

TC_EGRESS_FILTERS_WITHOUT_CHAIN = TC_EGRESS_FILTERS_BASE % {
    "chain_name": "",
    "filter_id1": FILETER_ID_1,
    "fip1": FLOATING_IP_1,
    "filter_id2": FILETER_ID_2,
    "fip2": FLOATING_IP_2}

TC_EGRESS_FILTERS_WITH_CHAIN = TC_EGRESS_FILTERS_BASE % {
    "chain_name": "chain 1 ",
    "filter_id1": FILETER_ID_1,
    "fip1": FLOATING_IP_1,
    "filter_id2": FILETER_ID_2,
    "fip2": FLOATING_IP_2}

INGRESS_QSIC_ID = "ffff:"
EGRESS_QDISC_ID = "1:"
QDISC_IDS = {constants.INGRESS_DIRECTION: INGRESS_QSIC_ID,
             constants.EGRESS_DIRECTION: EGRESS_QDISC_ID}
TC_QDISCS = [{'handle': '1:', 'qdisc_type': 'htb', 'parent': 'root'},
             {'handle': 'ffff:', 'qdisc_type': 'ingress', 'parent': 'ingress'}]

TC_FILTERS_ADD_BY_OTHERS_BASE = (
    'filter protocol %(protocol)s u32 %(chain_name)s \n'
    'filter protocol %(protocol)s u32 %(chain_name)s fh 800: ht divisor 1 \n'
    'filter protocol %(protocol)s u32 %(chain_name)s fh %(filter_id1)s order '
    '2048 key ht 800 bkt 0 flowid :1 not_in_hw  (rule hit 0 success 0) \n'
    'match %(matches)s at 16 (success 0 ) \n'
    'police 0x3 rate 3000Kbit burst 3Mb mtu 64Kb action drop overhead 0b '
    'linklayer ethernet \n'
    'ref 1 bind 1 installed 22 sec used 22 sec \n'
    '\n'
    ' Sent 111 bytes 222 pkts (dropped 0, overlimits 0)\n'
)

TC_FILTERS_ADD_BY_OTHERS_FOR_ALL_PROTOCOL = TC_FILTERS_ADD_BY_OTHERS_BASE % {
    "protocol": "all",
    "chain_name": "chain 1 ",
    "filter_id1": FILETER_ID_1,
    "matches": "c0a8dc06/ffffffff"
}


class TestFloatingIPTcCommandBase(base.BaseTestCase):
    def setUp(self):
        super(TestFloatingIPTcCommandBase, self).setUp()
        self.tc = tc_lib.FloatingIPTcCommandBase(
            FLOATING_IP_DEVICE_NAME,
            namespace=FLOATING_IP_ROUTER_NAMESPACE)
        self.execute = mock.patch('neutron.agent.common.utils.execute').start()

    def test__get_qdisc_id_for_filter(self):
        with mock.patch.object(base_tc_lib, 'list_tc_qdiscs',
                               return_value=TC_QDISCS):
            q1 = self.tc._get_qdisc_id_for_filter(constants.INGRESS_DIRECTION)
            self.assertEqual(INGRESS_QSIC_ID, q1)
            q2 = self.tc._get_qdisc_id_for_filter(constants.EGRESS_DIRECTION)
            self.assertEqual(EGRESS_QDISC_ID, q2)

    @mock.patch.object(base_tc_lib, 'add_tc_qdisc')
    def test__add_qdisc(self, mock_add_tc_qdisc):
        self.tc._add_qdisc(constants.INGRESS_DIRECTION)
        mock_add_tc_qdisc.assert_called_once_with(
            self.tc.name, 'ingress', namespace=self.tc.namespace)

        mock_add_tc_qdisc.reset_mock()
        self.tc._add_qdisc(constants.EGRESS_DIRECTION)
        mock_add_tc_qdisc.assert_called_once_with(
            self.tc.name, 'htb', parent='root', namespace=self.tc.namespace)

    def test__get_filters(self):
        self.tc._get_filters(INGRESS_QSIC_ID)
        self.execute.assert_called_with(
            ['ip', 'netns', 'exec', FLOATING_IP_ROUTER_NAMESPACE,
             'tc', '-p', '-s', '-d', 'filter', 'show', 'dev',
             FLOATING_IP_DEVICE_NAME,
             'parent', INGRESS_QSIC_ID, 'prio', 1],
            run_as_root=True,
            check_exit_code=True,
            log_fail_as_error=True,
            extra_ok_codes=None,
            privsep_exec=True
        )

    def _test__get_filterid_for_ip(self, filters):
        with mock.patch.object(tc_lib.FloatingIPTcCommandBase,
                               '_get_filters') as get_filters:
            get_filters.return_value = filters
            f_id = self.tc._get_filterid_for_ip(INGRESS_QSIC_ID, FLOATING_IP_1)
            self.assertEqual(FILETER_ID_1, f_id)

    def test__get_filterid_for_ip_without_chain(self):
        self._test__get_filterid_for_ip(TC_EGRESS_FILTERS_WITHOUT_CHAIN)

    def test__get_filterid_for_ip_with_chain(self):
        self._test__get_filterid_for_ip(TC_EGRESS_FILTERS_WITH_CHAIN)

    def test__get_filterid_for_ip_no_output(self):
        with mock.patch.object(tc_lib.FloatingIPTcCommandBase,
                               '_get_filters') as get_filters:
            get_filters.return_value = ""
            self.assertRaises(exceptions.FilterIDForIPNotFound,
                              self.tc._get_filterid_for_ip,
                              INGRESS_QSIC_ID, FLOATING_IP_1)

    def _test__get_filterid_for_ip_duplicated(self, filters):
        with mock.patch.object(tc_lib.FloatingIPTcCommandBase,
                               '_get_filters') as get_filters:
            get_filters.return_value = filters
            self.assertRaises(exceptions.MultipleFilterIDForIPFound,
                              self.tc._get_filterid_for_ip,
                              INGRESS_QSIC_ID, FLOATING_IP_2)

    def test__get_filterid_for_ip_duplicated_without_chain(self):
        self._test__get_filterid_for_ip_duplicated(
            TC_INGRESS_FILTERS_DUP_WITHOUT_CHAIN)

    def test__get_filterid_for_ip_duplicated_with_chain(self):
        self._test__get_filterid_for_ip_duplicated(
            TC_INGRESS_FILTERS_DUP_WITH_CHAIN)

    def _test__get_filterid_for_ip_not_found(self, filters):
        with mock.patch.object(tc_lib.FloatingIPTcCommandBase,
                               '_get_filters') as get_filters:
            get_filters.return_value = filters
            self.assertRaises(exceptions.FilterIDForIPNotFound,
                              self.tc._get_filterid_for_ip,
                              INGRESS_QSIC_ID, "1.1.1.1")

    def test__get_filterid_for_ip_not_found_without_chain(self):
        self._test__get_filterid_for_ip_not_found(
            TC_EGRESS_FILTERS_WITHOUT_CHAIN)

    def test__get_filterid_for_ip_not_found_with_chain(self):
        self._test__get_filterid_for_ip_not_found(TC_EGRESS_FILTERS_WITH_CHAIN)

    def test__get_filterid_for_ip_with_additional_filters(self):
        with mock.patch.object(tc_lib.FloatingIPTcCommandBase,
                               '_get_filters') as get_filters:
            get_filters.return_value = \
                TC_FILTERS_ADD_BY_OTHERS_FOR_ALL_PROTOCOL
            self.assertRaises(exceptions.FilterIDForIPNotFound,
                              self.tc._get_filterid_for_ip,
                              INGRESS_QSIC_ID, "192.168.220.6")

    def test__del_filter_by_id(self):
        self.tc._del_filter_by_id(INGRESS_QSIC_ID, FLOATING_IP_1)
        self.execute.assert_called_once_with(
            ['ip', 'netns', 'exec', FLOATING_IP_ROUTER_NAMESPACE,
             'tc', 'filter', 'del', 'dev', FLOATING_IP_DEVICE_NAME,
             'parent', INGRESS_QSIC_ID,
             'prio', 1, 'handle', FLOATING_IP_1, 'u32'],
            run_as_root=True,
            check_exit_code=True,
            log_fail_as_error=True,
            extra_ok_codes=None,
            privsep_exec=True
        )

    def _test__get_qdisc_filters(self, filters):
        with mock.patch.object(tc_lib.FloatingIPTcCommandBase,
                               '_get_filters') as get_filters:
            get_filters.return_value = filters
            f_ids = self.tc._get_qdisc_filters(INGRESS_QSIC_ID)
            self.assertEqual([FILETER_ID_1, FILETER_ID_2], f_ids)

    def test__get_qdisc_filters_without_chain(self):
        self._test__get_qdisc_filters(TC_EGRESS_FILTERS_WITHOUT_CHAIN)

    def test__get_qdisc_filters_with_chain(self):
        self._test__get_qdisc_filters(TC_EGRESS_FILTERS_WITH_CHAIN)

    def test__get_qdisc_filters_no_output(self):
        with mock.patch.object(tc_lib.FloatingIPTcCommandBase,
                               '_get_filters') as get_filters:
            get_filters.return_value = ""
            f_ids = self.tc._get_qdisc_filters(INGRESS_QSIC_ID)
            self.assertEqual(0, len(f_ids))

    def test__add_filter(self):
        protocol = ['protocol', 'ip']
        prio = ['prio', 1]
        match = ['u32', 'match', 'ip', 'dst', FLOATING_IP_1]
        police = ['police', 'rate', '1kbit', 'burst', '1kbit',
                  'mtu', '64kb', 'drop', 'flowid', ':1']
        args = protocol + prio + match + police
        cmd = ['tc', 'filter', 'add', 'dev', FLOATING_IP_DEVICE_NAME,
               'parent', INGRESS_QSIC_ID] + args

        self.tc._add_filter(INGRESS_QSIC_ID,
                            constants.INGRESS_DIRECTION,
                            FLOATING_IP_1, 1, 1)
        self.execute.assert_called_once_with(
            ['ip', 'netns', 'exec', FLOATING_IP_ROUTER_NAMESPACE] + cmd,
            run_as_root=True,
            check_exit_code=True,
            log_fail_as_error=True,
            extra_ok_codes=None,
            privsep_exec=True
        )

    def test__get_or_create_qdisc(self):
        with mock.patch.object(tc_lib.FloatingIPTcCommandBase,
                               '_get_qdisc_id_for_filter') as get_disc1:
            get_disc1.return_value = None
            with mock.patch.object(tc_lib.FloatingIPTcCommandBase,
                                   '_add_qdisc'):
                with mock.patch.object(
                        tc_lib.FloatingIPTcCommandBase,
                        '_get_qdisc_id_for_filter') as get_disc2:
                    get_disc2.return_value = INGRESS_QSIC_ID
                    qdisc_id = self.tc._get_or_create_qdisc(
                        constants.INGRESS_DIRECTION)
                    self.assertEqual(INGRESS_QSIC_ID, qdisc_id)

    def test__get_or_create_qdisc_failed(self):
        with mock.patch.object(tc_lib.FloatingIPTcCommandBase,
                               '_get_qdisc_id_for_filter') as get_disc1:
            get_disc1.return_value = None
            with mock.patch.object(tc_lib.FloatingIPTcCommandBase,
                                   '_add_qdisc'):
                with mock.patch.object(
                        tc_lib.FloatingIPTcCommandBase,
                        '_get_qdisc_id_for_filter') as get_disc2:
                    get_disc2.return_value = None
                    self.assertRaises(exceptions.FailedToAddQdiscToDevice,
                                      self.tc._get_or_create_qdisc,
                                      constants.INGRESS_DIRECTION)


class TestFloatingIPTcCommand(base.BaseTestCase):
    def setUp(self):
        super(TestFloatingIPTcCommand, self).setUp()
        self.tc = tc_lib.FloatingIPTcCommand(
            FLOATING_IP_DEVICE_NAME,
            namespace=FLOATING_IP_ROUTER_NAMESPACE)
        self.execute = mock.patch('neutron.agent.common.utils.execute').start()

    def _test_clear_all_filters(self, filters):
        with mock.patch.object(tc_lib.FloatingIPTcCommandBase,
                               '_get_qdisc_id_for_filter') as get_disc:
            get_disc.return_value = EGRESS_QDISC_ID
            with mock.patch.object(tc_lib.FloatingIPTcCommandBase,
                                   '_get_filters') as get_filters:
                get_filters.return_value = filters
                self.tc.clear_all_filters(constants.EGRESS_DIRECTION)
                self.assertEqual(2, self.execute.call_count)

    def test_clear_all_filters_without_chain(self):
        self._test_clear_all_filters(TC_EGRESS_FILTERS_WITHOUT_CHAIN)

    def test_clear_all_filters_with_chain(self):
        self._test_clear_all_filters(TC_EGRESS_FILTERS_WITH_CHAIN)

    def test_set_ip_rate_limit_filter_existed(self):
        with mock.patch.object(tc_lib.FloatingIPTcCommandBase,
                               '_get_qdisc_id_for_filter') as get_disc:
            get_disc.return_value = EGRESS_QDISC_ID
            with mock.patch.object(tc_lib.FloatingIPTcCommandBase,
                                   '_get_filterid_for_ip') as get_filter:
                get_filter.return_value = FILETER_ID_1
                with mock.patch.object(tc_lib.FloatingIPTcCommandBase,
                                       '_del_filter_by_id') as del_filter:
                    with mock.patch.object(tc_lib.FloatingIPTcCommandBase,
                                           '_add_filter') as add_filter:
                        ip = "111.111.111.111"
                        self.tc.set_ip_rate_limit(constants.EGRESS_DIRECTION,
                                                  ip, 1, 1)
                        del_filter.assert_called_once_with(
                            EGRESS_QDISC_ID, FILETER_ID_1)
                        add_filter.assert_called_once_with(
                            EGRESS_QDISC_ID, constants.EGRESS_DIRECTION,
                            ip, 1, 1)

    def _test_set_ip_rate_limit_no_qdisc(self, filters):
        with mock.patch.object(tc_lib.FloatingIPTcCommandBase,
                               '_get_qdisc_id_for_filter') as get_disc:
            get_disc.return_value = None
            with mock.patch.object(tc_lib.FloatingIPTcCommandBase,
                                   '_add_qdisc'):
                with mock.patch.object(tc_lib.FloatingIPTcCommandBase,
                                       '_get_filters') as get_filters:
                    get_filters.return_value = filters
                    get_disc.return_value = INGRESS_QSIC_ID
                    ip = "111.111.111.111"
                    self.tc.set_ip_rate_limit(constants.INGRESS_DIRECTION,
                                              ip, 1, 1)

                    protocol = ['protocol', 'ip']
                    prio = ['prio', 1]
                    _match = 'dst'
                    match = ['u32', 'match', 'ip', _match, ip]
                    police = ['police', 'rate', '1kbit', 'burst', '1kbit',
                              'mtu', '64kb', 'drop', 'flowid', ':1']
                    args = protocol + prio + match + police

                    self.execute.assert_called_once_with(
                        ['ip', 'netns', 'exec', FLOATING_IP_ROUTER_NAMESPACE,
                         'tc', 'filter', 'add', 'dev', FLOATING_IP_DEVICE_NAME,
                         'parent', INGRESS_QSIC_ID] + args,
                        run_as_root=True,
                        check_exit_code=True,
                        log_fail_as_error=True,
                        extra_ok_codes=None,
                        privsep_exec=True
                    )

    def test_set_ip_rate_limit_no_qdisc_without_chain(self):
        self._test_set_ip_rate_limit_no_qdisc(TC_INGRESS_FILTERS_WITHOUT_CHAIN)

    def test_set_ip_rate_limit_no_qdisc_with_chain(self):
        self._test_set_ip_rate_limit_no_qdisc(TC_INGRESS_FILTERS_WITH_CHAIN)

    def test_clear_ip_rate_limit(self):
        with mock.patch.object(tc_lib.FloatingIPTcCommandBase,
                               '_get_qdisc_id_for_filter') as get_disc:
            get_disc.return_value = EGRESS_QDISC_ID
            with mock.patch.object(tc_lib.FloatingIPTcCommandBase,
                                   '_get_filterid_for_ip') as get_filter_id:
                get_filter_id.return_value = FILETER_ID_1
                self.tc.clear_ip_rate_limit(constants.EGRESS_DIRECTION,
                                            FLOATING_IP_1)

                self.execute.assert_called_once_with(
                    ['ip', 'netns', 'exec', FLOATING_IP_ROUTER_NAMESPACE,
                     'tc', 'filter', 'del', 'dev', FLOATING_IP_DEVICE_NAME,
                     'parent', EGRESS_QDISC_ID,
                     'prio', 1, 'handle', FILETER_ID_1, 'u32'],
                    run_as_root=True,
                    check_exit_code=True,
                    log_fail_as_error=True,
                    extra_ok_codes=None,
                    privsep_exec=True
                )

    def test_get_filter_id_for_ip(self):
        with mock.patch.object(tc_lib.FloatingIPTcCommandBase,
                               '_get_qdisc_id_for_filter') as get_disc:
            get_disc.return_value = EGRESS_QDISC_ID
            with mock.patch.object(tc_lib.FloatingIPTcCommandBase,
                                   '_get_filterid_for_ip') as get_filter_id:
                self.tc.get_filter_id_for_ip(constants.EGRESS_DIRECTION,
                                             '8.8.8.8')
                get_filter_id.assert_called_once_with(EGRESS_QDISC_ID,
                                                      '8.8.8.8')

    def test_get_existing_filter_ids(self):
        with mock.patch.object(tc_lib.FloatingIPTcCommandBase,
                               '_get_qdisc_id_for_filter') as get_disc:
            get_disc.return_value = EGRESS_QDISC_ID
            with mock.patch.object(tc_lib.FloatingIPTcCommandBase,
                                   '_get_qdisc_filters') as get_filter_ids:
                self.tc.get_existing_filter_ids(constants.EGRESS_DIRECTION)
                get_filter_ids.assert_called_once_with(EGRESS_QDISC_ID)

    def test_delete_filter_ids(self):
        with mock.patch.object(tc_lib.FloatingIPTcCommandBase,
                               '_get_qdisc_id_for_filter') as get_disc:
            get_disc.return_value = EGRESS_QDISC_ID
            with mock.patch.object(tc_lib.FloatingIPTcCommandBase,
                                   '_del_filter_by_id') as del_filter_id:
                self.tc.delete_filter_ids(constants.EGRESS_DIRECTION,
                                          [FILETER_ID_1, FILETER_ID_2])
                del_filter_id.assert_has_calls(
                    [mock.call(EGRESS_QDISC_ID, FILETER_ID_1),
                     mock.call(EGRESS_QDISC_ID, FILETER_ID_2)])



# Copyright 2011 OpenStack Foundation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib.api import attributes
from neutron_lib.db import api as db_api
from neutron_lib.db import quota_api as nlib_quota_api
from neutron_lib import exceptions
from neutron_lib.plugins import constants
from neutron_lib.plugins import directory
from oslo_log import log

from neutron.db.quota import api as quota_api
from neutron.objects import quota as quota_obj
from neutron.quota import resource as res

LOG = log.getLogger(__name__)


class DbQuotaDriver(nlib_quota_api.QuotaDriverAPI):
    """Driver to perform necessary checks to enforce quotas and obtain quota
    information.

    The default driver utilizes the local database.
    """

    @staticmethod
    def get_default_quotas(context, resources, project_id):
        """Given a list of resources, retrieve the default quotas set for
        a project.

        :param context: The request context, for access checks.
        :param resources: A dictionary of the registered resource keys.
        :param project_id: The ID of the project to return default quotas for.
        :return: dict from resource name to dict of name and limit
        """
        # Currently the project_id parameter is unused, since all projects
        # share the same default values. This may change in the future so
        # we include project ID to remain backwards compatible.
        return dict((key, resource.default)
                    for key, resource in resources.items())

    @staticmethod
    @db_api.retry_if_session_inactive()
    def get_project_quotas(context, resources, project_id):
        """Given a list of resources, retrieve the quotas for the given
        project. If no limits are found for the specified project, the
        operation returns the default limits.

        :param context: The request context, for access checks.
        :param resources: A dictionary of the registered resource keys.
        :param project_id: The ID of the project to return quotas for.
        :return: dict from resource name to dict of name and limit
        """

        # init with defaults
        project_quota = dict((key, resource.default)
                             for key, resource in resources.items())

        # update with project specific limits
        quota_objs = quota_obj.Quota.get_objects(context,
                                                 project_id=project_id)
        for item in quota_objs:
            project_quota[item['resource']] = item['limit']

        return project_quota

    @db_api.retry_if_session_inactive()
    def get_detailed_project_quotas(self, context, resources, project_id):
        """Given a list of resources and a specific project, retrieve
        the detailed quotas (limit, used, reserved).
        :param context: The request context, for access checks.
        :param resources: A dictionary of the registered resource keys.
        :param project_id: The ID of the project to return quotas for.
        :return dict: mapping resource name in dict to its corresponding limit
            used and reserved. Reserved currently returns default value of 0
        """
        res_reserve_info = quota_api.get_reservations_for_resources(
            context, project_id, resources.keys())
        project_quota_ext = {}
        for key, resource in resources.items():
            if isinstance(resource, res.TrackedResource):
                used = self.get_resource_count(context, project_id, resource)
            else:
                # NOTE(ihrachys) .count won't use the plugin we pass, but we
                # pass it regardless to keep the quota driver API intact
                plugins = directory.get_plugins()
                plugin = plugins.get(key, plugins[constants.CORE])
                used = resource.count(context, plugin, project_id)

            project_quota_ext[key] = {
                'limit': resource.default,
                'used': used,
                'reserved': res_reserve_info.get(key, 0),
            }
        # update with specific project limits
        quota_objs = quota_obj.Quota.get_objects(context,
                                                 project_id=project_id)
        for item in quota_objs:
            project_quota_ext[item['resource']]['limit'] = item['limit']
        return project_quota_ext

    @staticmethod
    @db_api.retry_if_session_inactive()
    def delete_project_quota(context, project_id):
        """Delete the quota entries for a given project_id.

        After deletion, this project will use default quota values in conf.
        Raise a "not found" error if the quota for the given project was
        never defined.
        """
        if quota_obj.Quota.delete_objects(context, project_id=project_id) < 1:
            # No record deleted means the quota was not found
            raise exceptions.TenantQuotaNotFound(tenant_id=project_id)

    @staticmethod
    @db_api.retry_if_session_inactive()
    def get_all_quotas(context, resources):
        """Given a list of resources, retrieve the quotas for the all projects.

        :param context: The request context, for access checks.
        :param resources: A dictionary of the registered resource keys.
        :return: quotas list of dict of project_id:, resourcekey1:
        resourcekey2: ...
        """
        project_default = dict((key, resource.default)
                               for key, resource in resources.items())

        all_project_quotas = {}

        for quota in quota_obj.Quota.get_objects(context):
            project_id = quota['project_id']

            # avoid setdefault() because only want to copy when actually
            # required
            project_quota = all_project_quotas.get(project_id)
            if project_quota is None:
                project_quota = project_default.copy()
                project_quota['project_id'] = project_id
                attributes.populate_project_info(project_quota)
                all_project_quotas[project_id] = project_quota

            project_quota[quota['resource']] = quota['limit']

        # Convert values to a list to as caller expect an indexable iterable,
        # where python3's dict_values does not support indexing
        return list(all_project_quotas.values())

    @staticmethod
    @db_api.retry_if_session_inactive()
    def update_quota_limit(context, project_id, resource, limit):
        project_quotas = quota_obj.Quota.get_objects(
            context, project_id=project_id, resource=resource)
        if project_quotas:
            project_quotas[0].limit = limit
            project_quotas[0].update()
        else:
            quota_obj.Quota(context, project_id=project_id, resource=resource,
                            limit=limit).create()

    def _get_quotas(self, context, project_id, resources):
        """Retrieves the quotas for specific resources.

        A helper method which retrieves the quotas for the specific
        resources identified by keys, and which apply to the current
        context.

        :param context: The request context, for access checks.
        :param project_id: the project ID to check quota.
        :param resources: A dictionary of the registered resources.
        """
        # Grab and return the quotas (without usages)
        quotas = DbQuotaDriver.get_project_quotas(
            context, resources, project_id)

        return dict((k, v) for k, v in quotas.items())

    def _handle_expired_reservations(self, context, project_id):
        LOG.debug("Deleting expired reservations for project: %s", project_id)
        # Delete expired reservations (we don't want them to accrue
        # in the database)
        quota_api.remove_expired_reservations(context, project_id=project_id)

    @db_api.retry_if_session_inactive()
    def make_reservation(self, context, project_id, resources, deltas, plugin):
        # Lock current reservation table
        # NOTE(salv-orlando): This routine uses DB write locks.
        # These locks are acquired by the count() method invoked on resources.
        # Please put your shotguns aside.
        # A non locking algorithm for handling reservation is feasible, however
        # it will require two database writes even in cases when there are not
        # concurrent reservations.
        # For this reason it might be advisable to handle contention using
        # this kind of locks and paying the cost of a write set certification
        # failure when a MySQL Galera cluster is employed. Also, this class of
        # locks should be ok to use when support for sending "hotspot" writes
        # to a single node will be available.
        requested_resources = deltas.keys()
        with db_api.CONTEXT_WRITER.using(context):
            # "get_project_quotas" needs in input a dictionary mapping resource
            # name to BaseResource instances so that the default quota can be
            # retrieved
            current_limits = self.get_project_quotas(
                context, resources, project_id)
            unlimited_resources = set([resource for (resource, limit) in
                                       current_limits.items() if limit < 0])
            # Do not even bother counting resources and calculating headroom
            # for resources with unlimited quota
            LOG.debug("Resources %s have unlimited quota limit. It is not "
                      "required to calculate headroom ",
                      ",".join(unlimited_resources))
            requested_resources = (set(requested_resources) -
                                   unlimited_resources)
            # Gather current usage information
            # TODO(salv-orlando): calling get_resource_usage() for every
            # resource triggers multiple queries on quota usage. This should be
            # improved, however this is not an urgent matter as the REST API
            # currently only allows allocation of a resource at a time
            current_usages = dict(
                (resource, self.get_resource_usage(context, project_id,
                                                   resources, resource)) for
                resource in requested_resources)
            # Adjust for expired reservations. Apparently it is cheaper than
            # querying every time for active reservations and counting overall
            # quantity of resources reserved
            expired_deltas = quota_api.get_reservations_for_resources(
                context, project_id, requested_resources, expired=True)
            # Verify that the request can be accepted with current limits
            resources_over_limit = []
            for resource in requested_resources:
                expired_reservations = expired_deltas.get(resource, 0)
                total_usage = current_usages[resource] - expired_reservations
                res_headroom = current_limits[resource] - total_usage
                LOG.debug(("Attempting to reserve %(delta)d items for "
                           "resource %(resource)s. Total usage: %(total)d; "
                           "quota limit: %(limit)d; headroom:%(headroom)d"),
                          {'resource': resource,
                           'delta': deltas[resource],
                           'total': total_usage,
                           'limit': current_limits[resource],
                           'headroom': res_headroom})
                if res_headroom < deltas[resource]:
                    resources_over_limit.append(resource)
                if expired_reservations:
                    self._handle_expired_reservations(context, project_id)

            if resources_over_limit:
                raise exceptions.OverQuota(overs=sorted(resources_over_limit))
            # Success, store the reservation
            # TODO(salv-orlando): Make expiration time configurable
            return quota_api.create_reservation(
                context, project_id, deltas)

    def commit_reservation(self, context, reservation_id):
        # Do not mark resource usage as dirty. If a reservation is committed,
        # then the relevant resources have been created. Usage data for these
        # resources has therefore already been marked dirty.
        quota_api.remove_reservation(context, reservation_id,
                                     set_dirty=False)

    def cancel_reservation(self, context, reservation_id):
        # Mark resource usage as dirty so the next time both actual resources
        # used and reserved will be recalculated
        quota_api.remove_reservation(context, reservation_id,
                                     set_dirty=True)

    def limit_check(self, context, project_id, resources, values):
        """Check simple quota limits.

        For limits--those quotas for which there is no usage
        synchronization function--this method checks that a set of
        proposed values are permitted by the limit restriction.

        If any of the proposed values is over the defined quota, an
        OverQuota exception will be raised with the sorted list of the
        resources which are too high.  Otherwise, the method returns
        nothing.

        :param context: The request context, for access checks.
        :param project_id: The project ID to check the quota.
        :param resources: A dictionary of the registered resources.
        :param values: A dictionary of the values to check against the
                       quota.
        """

        # Ensure no value is less than zero
        unders = [key for key, val in values.items() if val < 0]
        if unders:
            raise exceptions.InvalidQuotaValue(unders=sorted(unders))

        # Get the applicable quotas
        quotas = self._get_quotas(context, project_id, resources)

        # Check the quotas and construct a list of the resources that
        # would be put over limit by the desired values
        overs = [key for key, val in values.items() if 0 <= quotas[key] < val]
        if overs:
            raise exceptions.OverQuota(overs=sorted(overs))

    @staticmethod
    def get_resource_usage(context, project_id, resources, resource_name):
        tracked_resource = resources.get(resource_name)
        if not tracked_resource:
            return

        return tracked_resource.count(context, None, project_id,
                                      resync_usage=False)

    @staticmethod
    def get_resource_count(context, project_id, tracked_resource):
        return tracked_resource.count_used(context, project_id,
                                           resync_usage=False)

    def quota_limit_check(self, context, project_id, resources, deltas):
        # Ensure no value is less than zero
        unders = [key for key, val in deltas.items() if val < 0]
        if unders:
            raise exceptions.InvalidQuotaValue(unders=sorted(unders))

        current_limits = self.get_project_quotas(context, resources,
                                                 project_id)
        overs = set()
        for resource_name, delta in deltas.items():
            resource_limit = current_limits.get(resource_name)
            if resource_limit in (None, quota_api.UNLIMITED_QUOTA):
                continue

            resource_usage = self.get_resource_usage(context, project_id,
                                                     resources, resource_name)
            if resource_usage is None:
                continue

            if resource_usage + delta > resource_limit:
                overs.add(resource_name)

        if overs:
            raise exceptions.OverQuota(overs=sorted(overs))

    @staticmethod
    def get_workers():
        return []



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from neutron.services.trunk.drivers import base


class FakeDriver(base.DriverBase):

    @property
    def is_loaded(self):
        return True

    @classmethod
    def create(cls):
        return cls('foo_name', ('foo_intfs',), ('foo_seg_types',))


class FakeDriver2(base.DriverBase):

    @property
    def is_loaded(self):
        return True

    @classmethod
    def create(cls):
        return cls('foo_name2', ('foo_intf2',), ('foo_seg_types2',))


class FakeDriverCanTrunkBoundPort(base.DriverBase):

    @property
    def is_loaded(self):
        return True

    @classmethod
    def create(cls):
        return cls('foo_name3', ('foo_intfs',),
                   ('foo_seg_types',), can_trunk_bound_port=True)


class FakeDriverWithAgent(base.DriverBase):

    @property
    def is_loaded(self):
        return True

    @classmethod
    def create(cls):
        return cls('foo_name4', ('foo_intfs',), ('foo_seg_types',), "foo_type")
